/*
 *  Alpha-Mini CCD Cam Driver
 *
 *  Copyright (C) 2000 Florin Boariu.
 *	
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Created: February 2000
 */

#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <errno.h>
#include <string.h>

#include <pgtools.h>

void sendmsg (const char *issue, const char *fmt, ...)
{
	va_list arg;
	
	if (issue) printf ("<pre>%s: ", issue);
	else printf ("<pre>");
	va_start (arg, fmt);
	vprintf (fmt, arg);
	va_end (arg);
	printf ("</pre>\n");
}

void pgopen (const char *title, const char *head)
{
	char buffer[MAXPGHEAD];
	char fname[1024];
	int fd, cnt=0;
	char *tpos, *hpos;
	char t[]="<!--the-title-->", h[]="<!--the-head-->";

	sprintf (fname, "%s/%s", TMPLDIR, "grey-begin.tmpl");
	
	if (-1 == (fd = open (fname, O_RDONLY))) {
		senderr ("pgopen: cannot open template %s: %s\n",
			 fname,
			 strerror (errno));
		exit (-1);
	}
	
	if (0 > (cnt = read (fd, buffer, MAXPGHEAD))) {
		senderr ("pgopen: cannot read template %s: %s\n",
			 fname, strerror (errno));
		exit (-1);
	}

	close (fd);

	buffer[cnt] = 0;

	/* fill in the head and the title */
	printf ("Content-type: text/html\n\n"); fflush (stdout);
	tpos = strstr (buffer, t);
	if (tpos) {
		write (1, buffer, tpos-buffer);  /* start..title */
		write (1, title, strlen(title)); /* title */
	} else tpos = buffer;

	hpos = strstr (tpos, h);
	if (hpos) {
		write (1, tpos, hpos-tpos);     /* title..head */
		write (1, head, strlen (head)); /* head */
		write (1, hpos, strlen (hpos)); /* head..end */
	} else write (1, tpos, strlen (tpos));  /* title..end */
}

void pgclose (void)
{
	char buffer[MAXPGTAIL];
	char fname[1024];
	int fd, cnt=0;

	sprintf (fname, "%s/%s", TMPLDIR, "grey-end.tmpl");
	
	if (-1 == (fd = open (fname, O_RDONLY))) {
		senderr ("pgclose: cannot open template %s: %s\n",
			 fname, strerror (errno));
		exit (-1);
	}
	
	if (0 > (cnt = read (fd, buffer, MAXPGTAIL))) {
		senderr ("pgclose: cannot read template %s: %s\n",
			 fname, strerror (errno));
		exit (-1);
	}

	close (fd);

	buffer[cnt] = 0;

	fflush (stdout);
	fflush (stderr);
	
	write (1, buffer, cnt);
}

void sendfile (const char *name)
{
	char buffer[1024];
	char fname[1024];
	char null=0;
	int fd, cnt=0, total=0;

	sprintf (fname, "%s/%s", BODYDIR, name);
	
	if (-1 == (fd = open (fname, O_RDONLY))) {
		senderr ("sendfile: cannot open %s: %s\n",
			 fname, strerror (errno));
		exit (-1);
	}
	
	do {
		if (0 > (cnt = read (fd, buffer, 1024))) {
			senderr ("sendfile: cannot read %s: %s\n",
				 fname, strerror (errno));
			exit (-1);
		}
		total+= cnt;

		write (1, buffer, cnt);

	} while (cnt > 0);

	write (1, &null, 1);

	close (fd);
}
