/*
 *  Alpha-Mini CCD Cam Driver -- amin.c
 *
 *  Copyright (C) 2000 Florin Boariu.
 *	
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Created: January 2000
 */

#include <linux/amin.h>

/* module header files */
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/sys.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/config.h>
//#include <asm/unistd.h>

#include <linux/module.h>

/* driver header files */
#include <linux/fs.h>
#include <linux/devfs_fs_kernel.h>

#include <linux/delay.h>
#include <asm/io.h>
#include <linux/string.h>

#include <asm/uaccess.h>

static devfs_handle_t devfs_handle=NULL;

/*
 * HARDWARE ACCESS ROUTINES BEGIN
 */

typedef struct _amin_port_mirror {
	unsigned char dataReg, statReg, ctrlReg;
} amin_port_mirror;

#define CAM_PORT 0x378
#define DATA_REG 0x378 //(CAM_PORT)
#define STAT_REG 0x379 //(CAM_PORT+1)
#define CTRL_REG 0x37a //(CAM_PORT+2)

static int AminPort = CAM_PORT;
/* static int AminDataReg = AminPort + 0x00;
   static int AminStatReg = AminPort + 0x01;
   static int AminCtrlReg = AminPort + 0x02; */

#define AminDataReg (AminPort)
#define AminStatReg (AminPort+1)
#define AminCtrlReg (AminPort+2)

/*
 * Cam Status
 */

static int AminTimeOut=500000, AminCnt;  /* timeout counters */
unsigned int AminAux;
int AminXPos, AminYPos;
static struct _amin_cam_info AminInfo; /* setup and state */

struct _amin_kbuf amin_setup_buf, amin_state_buf;

/* mirror of the port */
static amin_port_mirror pmirror = {0, 0, 0};

/* new stuff for alpha-mini */

/* start - line */
#define START_LOW       pmirror.ctrlReg |= 0x01; outb (pmirror.ctrlReg, AminCtrlReg)
#define START_HIGH      pmirror.ctrlReg &= 0xfe; outb (pmirror.ctrlReg, AminCtrlReg)

/* mode - line */
#define MODE_LOW        pmirror.dataReg &= 0xfe; outb (pmirror.dataReg, AminDataReg)
#define MODE_HIGH       pmirror.dataReg |= 0x01; outb (pmirror.dataReg, AminDataReg)

/* parallel line macros */
#define WAIT_PSLOW                                           \
{                                                            \
        AminCnt = AminTimeOut;                               \
	while ( !(inb(AminStatReg)&0x80) && (--AminCnt) ) {} \
}

#define WAIT_PSLOW_T(m)                                      \
{                                                            \
        AminCnt = 1000;                                      \
	while ( !(inb(AminStatReg)&0x80) && (--AminCnt) ) {  \
		if (AminCnt < 5) {                           \
			AminCnt = 1000;                      \
			udelay (950);                        \
			if (!--m) AminCnt=1;                 \
		}                                            \
	}                                                    \
}

#define WAIT_PSLOW_W                          \
{                                             \
	while ( !(inb(AminStatReg)&0x80) ) {} \
}

#define WAIT_TIMEOUT       (AminCnt == 0)
#define CLEAR_TIMEOUT      (AminCnt = AminTimeOut)
#define CHECK_TIMEOUT(ret)                            \
{                                                     \
	if (WAIT_TIMEOUT) {                           \
		printk ("amin: timeout!\n");          \
		amin_unreset_cam (&(AminInfo.state)); \
		return (ret);                         \
	}                                             \
	CLEAR_TIMEOUT;                                \
}

#define CHECK_TIMEOUT_STI(ret)                        \
{                                                     \
	if (WAIT_TIMEOUT) {                           \
		printk ("amin: timeout!\n");          \
		amin_unreset_cam (&(AminInfo.state)); \
		sti ();                               \
		return (ret);                         \
	}                                             \
	CLEAR_TIMEOUT;                                \
}

#define WAIT_PSHIGH                                          \
{                                                            \
        AminCnt = AminTimeOut;                               \
	while ( (inb(AminStatReg)&0x80) && (--AminCnt) ) {}  \
}

#define WAIT_PSHIGH_T(m)                                     \
{                                                            \
        AminCnt = 1000;   /* that's _very_ fast */           \
	while ( (inb(AminStatReg)&0x80) && (--AminCnt) ) {   \
		if (AminCnt < 5) {                           \
			AminCnt = 1000;                      \
			udelay (950); /* wait about 1ms */   \
			if (!--m) AminCnt=1;                 \
		}                                            \
	}                                                    \
}

#define WAIT_PSHIGH_W                         \
{                                             \
	while ( (inb(AminStatReg)&0x80) ) {}  \
}

/* i/o */

void dela (long time)
{
	for (;--time;)
		udelay (950); /* very elegant, but very unprecise. however... */
}

#define START_PULSE        START_LOW; WAIT_PSLOW; if (!WAIT_TIMEOUT) { START_HIGH; WAIT_PSHIGH; }
#define START_PULSE_W      START_LOW; WAIT_PSLOW_W; START_HIGH; WAIT_PSHIGH;

#define MODE_PULSE         MODE_LOW; WAIT_PSLOW;      if (!WAIT_TIMEOUT) { MODE_HIGH;  WAIT_PSHIGH; }
#define MODE_PULSE_T(m)    MODE_LOW; WAIT_PSLOW_T(m); if (!WAIT_TIMEOUT) { MODE_HIGH;  WAIT_PSHIGH; }
#define MODE_PULSE_W       MODE_LOW; WAIT_PSLOW_W;    MODE_HIGH;  WAIT_PSHIGH;

#define RECV_BYTE(b)                                \
{                                                   \
	CLEAR_TIMEOUT;                              \
	START_LOW;                                  \
	WAIT_PSLOW;                                 \
	if (!WAIT_TIMEOUT) {                        \
		b = ((inb(AminStatReg)>>3) & 0x0f); \
		START_HIGH;                         \
		WAIT_PSHIGH;                        \
	}                                           \
}

#define SEND_BYTE(b)                                                   \
{                                                                      \
        CLEAR_TIMEOUT;                                                 \
	for (AminAux=0; (AminAux<=b) && !(WAIT_TIMEOUT); AminAux++) {  \
		MODE_PULSE;                                            \
	}                                                              \
	START_PULSE;                                                   \
}

#define SEND_BYTE_T(b,mul)                                             \
{                                                                      \
        CLEAR_TIMEOUT;                                                 \
	for (AminAux=0; (AminAux<=b) && !(WAIT_TIMEOUT); AminAux++) {  \
		MODE_PULSE_T(mul);                                     \
	}                                                              \
	START_PULSE;                                                   \
}

/*
 * ...W-Routines might block the system! They must be replaced!
 */
#define SEND_BYTE_W(b)                              \
{                                                   \
	for (AminAux=0; (AminAux<=b); AminAux++) {  \
		MODE_PULSE_W;                       \
	}                                           \
	START_PULSE;                                \
}

/*
 * send and receive whole strings
 *
 * buf:  the buffer (unsigned char preferred)
 * max:  maximum bytes to write in the buffer
 */
#define SEND_BUF(buf,max)               \
{                                       \
        unsigned int bgn=0, end=max;    \
	for (; bgn < end; bgn++) {      \
                SEND_BYTE ((buf[bgn])); \
	}                               \
}

#define RECV_BUF(buf,max)                   \
{                                           \
        unsigned int bgn=0, end=max;        \
        unsigned char aux=0x00;             \
	for (; bgn < end; bgn++) {          \
                RECV_BYTE ((buf[bgn]),aux); \
	}                                   \
}




/*
 * HARDWARE ACCESS ROUTINES END
 */

#define SUCCESS 0

#define DEVICE_NAME "amin"

static int AminDeviceOpen = 0;
static int AminMajor = 110;


/*
 * amin_open: called when the device is opened
 */
static int amin_open (struct inode *inode, struct file *file)
{
	/*printk ("amin: open (%p, %p)\n", inode, file);
	  printk ("amin: device is %d:%d\n",
	  inode->i_rdev>>8, inode->i_rdev & 0xff);*/

	if (AminDeviceOpen) return -EBUSY;

	AminDeviceOpen++;

#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
       
	return SUCCESS;
}

/*
 * amin_release: called when the device is closed
 */
static int amin_release (struct inode *inode, struct file *file)
{
	/*printk ("amin: release (%p, %p)\n", inode, file);*/
	
	AminDeviceOpen--;

#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif

	return 0;
}

/*
 * amin_read
 */
static ssize_t amin_read (struct file *file, char *buffer, size_t count, loff_t *offset)
{
	unsigned char min=MINOR(file->f_dentry->d_inode->i_rdev);

	switch (min) {
	case AMIN_MIN_RAW8:  /* image file -- 8 bpp */
		return amin_read_img (buffer, count, 8);

	case AMIN_MIN_RAW16: /* image file -- 12/16 bpp */
		return amin_read_img (buffer, count, 16);

	case AMIN_MIN_SETUP: /* cam setup info */
		return amin_read_kbuf (buffer, count, &amin_setup_buf);

	case AMIN_MIN_STATE: /* cam state info */
		return amin_read_kbuf (buffer, count, &amin_state_buf);
		
	default:
		return 0;
	}
}

struct file_operations AminFops = {
#ifdef MODULE
	owner: THIS_MODULE,
#endif MODULE
	read:    (void *)amin_read,
	write:   (void *)amin_write,
	ioctl:   (void *)amin_ioctl,
	open:    (void *)amin_open,
	release: (void *)amin_release /* a.k.a. close */
};

/* init the module */

#ifdef MODULE
#define amin_init init_module
#endif

int amin_init (void)
{
	int err, i, max;

	printk ("amin: OES Alpha-Mini CCD Cam driver v0.1.99\n");
	printk ("amin: Written by Florin Boariu <florin@bnv-bamberg.de>, 2000-2001\n");

	/* bzero-replacement */
	max = sizeof (struct _amin_cam_info);
	for (i=0; i < max; i++) ((unsigned char*)&AminInfo)[i] = 0;
	
	err = devfs_register_chrdev (AminMajor,
				     DEVICE_NAME,
				     &AminFops);

	if (err < 0) {
		printk ("amin: registration failed (major %d, error code %d)\n", 
			AminMajor, err);
		return err;
	}
	
	printk ("amin: registering device with major %d\n", AminMajor);

	devfs_handle = devfs_mk_dir (NULL, "amin", NULL);
	devfs_register (devfs_handle, "raw8", DEVFS_FL_DEFAULT, AminMajor, AMIN_MIN_RAW8,
			S_IFCHR | S_IRUGO | S_IWUGO, &AminFops, NULL);
	devfs_register (devfs_handle, "raw12", DEVFS_FL_DEFAULT, AminMajor, AMIN_MIN_RAW12,
			S_IFCHR | S_IRUGO | S_IWUGO, &AminFops, NULL);
	devfs_register (devfs_handle, "raw16", DEVFS_FL_DEFAULT, AminMajor, AMIN_MIN_RAW16,
			S_IFCHR | S_IRUGO | S_IWUGO, &AminFops, NULL);
	devfs_register (devfs_handle, "setup", DEVFS_FL_DEFAULT, AminMajor, AMIN_MIN_SETUP,
			S_IFCHR | S_IRUGO, &AminFops, NULL);
	devfs_register (devfs_handle, "state", DEVFS_FL_DEFAULT, AminMajor, AMIN_MIN_STATE,
			S_IFCHR | S_IRUGO, &AminFops, NULL);
	
	amin_print_info (&AminInfo);
	amin_mksetupbuf (&(AminInfo.setup), &amin_setup_buf);
	amin_mkstatebuf (&(AminInfo.state), &amin_state_buf);
  
	return 0;
}

/* cleanup */
void cleanup_module (void)
{
	int ret;
	
	devfs_unregister (devfs_handle);
	ret = devfs_unregister_chrdev (AminMajor, DEVICE_NAME);
	
	if (ret < 0) printk ("amin: error while unregistering (%d)\n", ret);
}

static int amin_read_kbuf (char *buffer, int count, amin_kbuf *b)
{
	int maxlen=0;

	if (b->pos < b->len) {
		maxlen = b->len-b->pos;
		if (maxlen > count) maxlen = count;
		copy_to_user (buffer, &(b->buf[b->pos]), maxlen);
		b->pos += maxlen;
	} else {
		b->pos = 0; /* make the file readable more than once */
	}

	return maxlen;
}

static int amin_read_img (char *buffer, int count, int bpp)
{
	int bytes_read=0;
	int i;
	unsigned char hn, mn, ln;
	
	if (!AminInfo.state.lite)
		amin_lite_frame (&AminInfo, 1);

	cli ();

	printk ("|");

	/* check if liting time is up (the cam won't accept any cmds if it's not */
	amin_sync (AminInfo.setup.time);
	
	printk (">");
	
	if (!AminInfo.state.hclip) {
		/* clip away the first lines (0..ystart) */
		for (i=0; i < AminInfo.setup.ystart; i++) {
			SEND_BYTE (AMIN_MOVELINE); CHECK_TIMEOUT_STI (-EIO);	      
		}
		SEND_BYTE (AMIN_SENDLINE); CHECK_TIMEOUT_STI (-EIO);
		for (i=0; i < AminInfo.state.maxX; i++) {
			RECV_BYTE (hn);    CHECK_TIMEOUT_STI (-EIO);
			RECV_BYTE (mn);    CHECK_TIMEOUT_STI (-EIO);
			RECV_BYTE (ln);    CHECK_TIMEOUT_STI (-EIO);
		}
		AminInfo.state.hclip = 1;
	}

	for (; AminInfo.state.lastY < AminInfo.state.maxY; AminInfo.state.lastY++) {
		
		/* next line won't fit in the buffer */
		if (bytes_read+AminInfo.state.maxX*2 >= count) goto done;

		/* vertical binning */
		for (i = 0; i < (1<<AminInfo.setup.ybin); i++) {
			SEND_BYTE (AMIN_MOVELINE);  CHECK_TIMEOUT_STI (-EIO);
		}

		SEND_BYTE (AMIN_SENDLINE); CHECK_TIMEOUT_STI (-EIO);
		for (; AminInfo.state.lastX < AminInfo.state.maxX; AminInfo.state.lastX++) {
			RECV_BYTE (hn);    CHECK_TIMEOUT_STI (-EIO);
			RECV_BYTE (mn);    CHECK_TIMEOUT_STI (-EIO);
			RECV_BYTE (ln);    CHECK_TIMEOUT_STI (-EIO);
			/* write data to user mem */
			if (bpp <= 8) {
				put_user ((hn<<4)+mn,  (unsigned char*)&(buffer[bytes_read]));
				bytes_read++;
			} else {
				put_user (hn, (unsigned char*)&(buffer[bytes_read]));
				put_user ((mn<<4)+ln, (unsigned char*)&(buffer[bytes_read]));
				bytes_read+=2;;
			}
		}
		AminInfo.state.lastX = 0;
	}

done:
	sti ();
	printk ("amin: %d bytes read\n", bytes_read);

	return bytes_read;
}

/*
 * amin_write
 */

#ifndef BEGINS
#define BEGINS(var,text) ((unsigned char*)var==(unsigned char*)strstr(var,text))
#endif
static ssize_t amin_write (struct file *file, char *buffer, size_t size, loff_t *offset)
{
	char text[1024];
	char *fsbuf=buffer;
	int buflen=size, cnt=0, p;
	unsigned long int i1, i2, i3, i4;	

	while ( (p=next_fs_block (&fsbuf, &buflen, text, 1024)) ) {
		cnt += p;
		printk ("amin: write: block is \"%s\"\n", text);
		
		if ( BEGINS (text,"reset-cam")) {
			i1 = getint (text, "lotime", 10);
			i2 = getint (text, "hitime", 30);
			printk ("amin: write: reset_cam (%d, %d)\n", 
				(unsigned)i1, (unsigned)i2);
			amin_reset_cam (&(AminInfo.state), i1, i2);

		} else if ( BEGINS (text, "init-cam")) {
			/* region */
			i2 = getint (text, "top", 0);
			i1 = getint (text, "left", 0);
			i3 = getint (text, "width", AMIN_PHYS_WIDTH-i1);
			i4 = getint (text, "height", AMIN_PHYS_HEIGHT-i2);
			printk ("amin: write: set_region (%d, %d, %d, %d)\n",
				(unsigned)i1, (unsigned)i2, 
				(unsigned)i3, (unsigned)i4);
			amin_set_region (&(AminInfo), i1, i2, i3, i4);

			/* time */
			i1 = getint (text, "time", 1000);
			printk ("amin: write: set_time (%d)\n", (unsigned)i1);
			amin_set_time (&(AminInfo), i1);

			/* binning */
			i1 = getint (text, "xbin", 0x00);
			i2 = getint (text, "ybin", 0x00);
			printk ("amin: write: set_binning (%d, %d)\n", 
				(unsigned)i1, (unsigned)i2);
			amin_set_binning (&AminInfo, i1, i2);
			
			amin_init_cam (&AminInfo);

		} else if ( BEGINS (text, "lite-frame")) {
			i1 = getint (text, "frame", 1);
			printk ("amin: write: lite_frame (%d)\n", (unsigned)i1);
			amin_lite_frame (&AminInfo, (unsigned)i1);

		} else if ( BEGINS (text, "print-info")) {
			amin_print_info (&AminInfo);

		} else if ( BEGINS (text, "wait-nop")) {
			printk ("amin: saw \"wain-nop\" -- blocking now\n");
			SEND_BYTE_W (AMIN_NOP);
			printk ("amin: we're back!\n");

		}
	}
	
	return cnt;
}

static int amin_ioctl (struct inode *inode, struct file *file,
		       unsigned int cmd, void *arg)
{
	return -EIO;
}

/*
 * converts ASCII string into an integer
 * the string is supposes to be [+-][0..9] (so decimal)
 */

int strtoint (const char *str)
{
	int nr=0, mul=1;

	if (!str) return 0;

	if (*str == '+') str++;
	if (*str == '-') mul=-1;
	
	for (; *str >= '0' && *str <= '9' && *str; str++)
		nr = (nr*10)+(*str)-(int)'0';
	
	return nr*mul;
}

/*
 * splits a buffer into blocks (delimiters are SPACEs, NEWLINEs and TABs) 
 * the routine might be a little lame, but... it works, and speed is not
 * essential in this case.
 * second thought: it sucks.
 */
int next_fs_block (char **buf, int *len, char *next, int nextlen)
{
	int cnt=0, i=0;
	unsigned char c=0;

	/* jump over initial spaces etc. */
	get_user (c, *buf);
	for (; (c==' ' || c<31 || c>128) && (*len); (*buf)++, cnt++, (*len)--);

	/* read valid chars */	
	get_user(c, *buf);
	for (;	(c!=' ' && c>31 && c<128) && (*len) && (i<nextlen); 
	     (*buf)++, cnt++, (*len)--, i++) {
		next[i]=c;
		get_user (c, *buf);
	}

	next[i<nextlen?i:nextlen] = 0;

	return cnt;
}

inline long int getint (const char *text, const char *key, long int def)
{
	const char *tmp = findval (text, key);
	return tmp ? strtoint (tmp) : def;
}

inline const char *findval (const char *var, const char *key)
{
	const char *val;
	if (!var || !key) return 0;
	val = (const char*)strstr(var,key);
	if (!val) return 0;
	while (*val) { if ((*(val++))=='=') return val; }
	return 0;
}

/* cam control */

/*
 * sends a NOP to the cam and waits for response
 * a (limited) time of minimum 'msec' msecs
 *
 * it works by sending a NOP pulse and waiting
 * for answer a 1000-counter-loop. If no answer
 * arrives, a udelay of 950us (that's about 1ms)
 * occures, then another loop. It might not be the
 * best method to do so (if someone knows a better
 * one, please mail), but it seems to work
 */
inline void amin_sync (unsigned long msec)
{
	SEND_BYTE_T (AMIN_NOP, msec);
}

/* sends RESET to the cam */
void amin_reset_cam (amin_cam_state *stat, int lo_time, int hi_time)
{
	cli();

	START_LOW;
	MODE_LOW;
	dela (lo_time);
	START_HIGH;
	MODE_HIGH;
	dela (hi_time);

	sti ();

	stat->reset = 1;
	stat->init  = 0;
	stat->lite  = 0;
	stat->frame = 0;
	stat->maxX  = 0;
	stat->maxY  = 0;
	stat->hclip = 0;
	stat->lastX = 0;
	stat->lastY = 0;

	amin_mkstatebuf (stat, &amin_state_buf);
}

/* 
 * sets state.reset to ZERO, so that on next LITE, a
 * RESET will occur
 */
void amin_unreset_cam (amin_cam_state *stat)
{
	stat->reset = 0;
	stat->init = 0;
	stat->lite = 0;
	stat->frame = 0;
	amin_mkstatebuf (stat, &amin_state_buf);
}

/* sets the binning in the SETUP-Struct */
void amin_set_binning (amin_cam_info *info, int xbin, int ybin)
{

	if (xbin >= 0 && xbin <= 0x03)
		info->setup.xbin = xbin;
	else info->setup.xbin = 0;
	if (ybin >= 0 && ybin <= 0x03)
		info->setup.ybin = ybin;
	else info->setup.ybin = 0;

	amin_uninit_cam (&(info->state));
}

/* sets the logical size of the image */
void amin_set_region (amin_cam_info *info, int x, int y, int w, int h)
{
	info->setup.left   = x/8 + (x%8 != 0); /* round up to 8 pixels */
	info->setup.mid    = w/8 + (w%8 != 0); /* round up to 8 pixels */
	info->setup.right  = AMIN_PHYS_WIDTH/8 -
		(info->setup.left+info->setup.mid);
	info->setup.ystart = y;
	info->setup.ysize  = h;

	amin_uninit_cam (&(info->state));

	/* do we need to set this? actually we don't... */
	info->state.maxX = w / (1<<(info->setup.xbin));
	info->state.maxY = h / (1<<(info->setup.ybin));
}

/* sets the liting time */
void amin_set_time (amin_cam_info *info, unsigned long int msecs)
{
	amin_uninit_cam (&(info->state));
	info->setup.time = msecs;
}

/* sends INIT to the cam */
int amin_init_cam (amin_cam_info *info)
{
	unsigned char t1, t2, t3, t4;

	if (!info->state.reset)
		amin_reset_cam (&(info->state), 10, 30);

	amin_check_setup (&(info->setup));	

	/* !! Intel Specific (endianess!) */
	t1 =  info->setup.time;
	t2 = (info->setup.time >> 8);
	t3 = (info->setup.time >> 16);
	t4 = (info->setup.time >> 24);
	
	printk ("amin: time: %x %x %x %x\n", t1, t2, t3, t4);

	printk ("amin: init-cam: ");

	/* start init phase of the cam */
	SEND_BYTE (AMIN_INIT);             CHECK_TIMEOUT (-EIO);  printk (".");

	/* receive version number of the hardware */
	RECV_BYTE (info->setup.version);   CHECK_TIMEOUT (-EIO);  printk (".");

	/* send initialization data */
	SEND_BYTE (info->setup.xbin);      CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (info->setup.left);      CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (info->setup.mid);       CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (info->setup.right);     CHECK_TIMEOUT (-EIO);  printk (".");

	/* !! WATCH IT! Will work only for little endians (i think) */

	/*SEND_BUF ((char*)(&(info->setup.time)), 4);
	CHECK_TIMEOUT (-EIO);  printk (".");*/

	SEND_BYTE (t1);                    CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (t2);                    CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (t3);                    CHECK_TIMEOUT (-EIO);  printk (".");
	SEND_BYTE (t4);                    CHECK_TIMEOUT (-EIO);  printk (".");
	
	if (info->setup.version != AMIN_REQ_VERSION) {
		printk ("amin: wrong hardware version (%d, requested: %d)\n",
			info->setup.version, AMIN_REQ_VERSION);
		return -EIO;
	}

	info->state.init = 1;
	info->state.lite = 0;
	info->state.frame = 0;
	info->state.maxX  = (info->setup.mid*8) / (1<<(info->setup.xbin));
	info->state.maxY  = (info->setup.ysize) / (1<<(info->setup.ybin));
	info->state.lastX = 0;
	info->state.lastY = 0;

	amin_mksetupbuf (&(info->setup), &amin_setup_buf);

	printk (" success.\n");

	return 0;
}

void amin_sfcpy (char *tgt, const char *fmt, ...)
{	
}

void amin_mksetupbuf (amin_cam_setup *setup, amin_kbuf *b)
{

	sprintf (b->buf, "hw_version=%d\n"
		 "left8=%d\nmid8=%d\nright8=%d\nystart=%d\ntime=%u\n"
		 "xbin=%d\nybin=%d\n"
		 "log_width=%d\nlog_height=%d\n",
		 setup->version,
		 setup->left,
		 setup->mid,
		 setup->right,
		 setup->ystart,
		 (unsigned)setup->time,
		 1<<setup->xbin,
		 1<<setup->ybin,
		 (setup->mid*8)/(1<<setup->xbin),
		 (setup->ysize)/(1<<setup->ybin));
	b->len = strlen(b->buf); /* \0 counts too */
	b->pos = 0;
	b->buf[b->len] = 0;
}

void amin_mkstatebuf (amin_cam_state *state, amin_kbuf *b)
{
	sprintf (b->buf, "reset=%s\ninit=%s\nlite=%s\n"
		  "frame=%d\nxmax=%d\nymax=%d\n"
		  "xlast=%d\nylas=%d\n",
		  state->reset ? "true" : "false",
		  state->init  ? "true" : "false",
		  state->lite  ? "true" : "false",
		  state->frame, state->maxX, state->maxY,
		  state->lastX, state->lastY);
	b->len = strlen(b->buf)+1; /* \0 counts too */
	b->pos = 0;
	b->buf[b->len] = 0;
}

/* uninits the cam (sets the STATUS-struct to zero, except for the RESET-field) */
void amin_uninit_cam (amin_cam_state *stat)
{
	stat->init = 0;
	stat->lite = 0;
	stat->frame = 0;

	amin_mkstatebuf (stat, &amin_state_buf);
}

/*
 * lites the respective frame -- it WON'T WAIT until the cam finishes!
 * The cam won't answer until the liting time is up, but this function
 * won't wait that long, it will return as soon as the command has been
 * sent. The idea behind this is that we don't want to block the
 * system and we leave it to the application programmer to call amin_nop
 * shortly before the time is up, and right after that amin_read.
 * amin_nop will block until the cam responses again, while amin_read
 * will read the image data.
 * PS: amin_nop might be called differently in the future
 *     (amin_block? amin_sync?)
 */
int amin_lite_frame (amin_cam_info *info, int frame)
{
	if (!(info->state.init)) {
		amin_init_cam (info);
	}

	printk ("amin: liting ");
	
	if (frame != 2) {
		if (frame != 1) {
			printk ("amin: frame is %d, requested are 1 or 2. Assuming 1.\n", 
				frame);
		}
		printk ("frame 1... ");
		SEND_BYTE (AMIN_FRAME1);
		CHECK_TIMEOUT (-EIO);
		info->state.frame = 1;
	} else {
		printk ("fame 2... ");
		SEND_BYTE (AMIN_FRAME2);
		CHECK_TIMEOUT (-EIO);
		info->state.frame = 2;
	}

	info->state.lite = 1;
	info->state.lastX = 0;
	info->state.lastY = 0;
	info->state.init = 0;
	info->state.hclip = 0;

	amin_mkstatebuf (&(info->state), &amin_state_buf);

	printk ("done.\n");

	return 0;
}

/* 
 * check the validity of the SETUP-Struct and
 * set the defaults where values are missing
 */
void amin_check_setup (amin_cam_setup *setup)
{
	/* vertical metrics */

	if ((setup->ystart < 0) || (setup->ystart > AMIN_PHYS_HEIGHT)) {
		setup->ystart = 0;
	}

	if (setup->ybin > 0x03) setup->ybin    = 0x00;
	
	if ((setup->ysize < 1) || 
	    (setup->ysize > AMIN_PHYS_HEIGHT-setup->ystart)) {
		setup->ysize = AMIN_PHYS_HEIGHT-setup->ystart;
	}
	
	/* make sure the height is a multiple of the binning value */
	if (setup->ysize % (1<<setup->ybin)) 
		setup->ysize = (setup->ysize/(1<<setup->ybin))*(1<<setup->ybin);


	/* horizontal metrics */
	if (setup->xbin > 0x03)               setup->xbin    = 0x00;
	if (setup->left > AMIN_PHYS_WIDTH/8)  setup->left    = 0; 
	if (setup->right > (AMIN_PHYS_WIDTH/8-setup->left))
		setup->right   = 0;
	setup->mid = AMIN_PHYS_WIDTH/8 - (setup->left+setup->right);

	setup->version = 0;
	if (!setup->time) setup->time = 100;
}

/*
 * prints the struct on the screen -- for debugging purposes
 */
void amin_print_info (amin_cam_info *info)
{
	printk ("amin: -----------------------------------------------------------------\n");
	printk ("amin: setup: \tystart=%d  ysize=%d\n", 
		info->setup.ystart, info->setup.ysize);
	printk ("amin: \t\tbinning=%d  left=%d  mid=%d  right=%d\n", 
		info->setup.xbin,
		info->setup.left,
		info->setup.mid, 
		info->setup.right);
	printk ("amin: \t\tversion=%d\n",
		info->setup.version);
	printk ("amin: \t\ttime=%d\n",
		(unsigned)info->setup.time);

	printk ("amin: state: \treset=%d  init=%d  lite=%d  frame=%d\n",
		info->state.reset,
		info->state.init,
		info->state.lite,
		info->state.frame);
	printk ("amin: \t\tmaxX=%d \tmaxY=%d\n",
		info->state.maxX,
		info->state.maxY);
	printk ("amin: \t\tlastX=%d \tlastY=%d\n",
		info->state.lastX,
		info->state.lastY);
	printk ("amin: -----------------------------------------------------------------\n");
}





