#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

/*
 * This program has been copied from the libjpeg-6b sources
 * and adjusted. The copyrights remain with the original
 * author. See the libjpeg license for details.
 */

#include <stdio.h>
#include "jpeglib.h"

#include <setjmp.h>

JSAMPLE * image_buffer;  /* Points to large array of data */
int image_height;        /* Number of rows in image */
int image_width;         /* Number of columns in image */

/*
 * Watch it! Will "eat" only grayscales! (1 color plane)
 */

GLOBAL(void)
write_JPEG_file (char * filename, int quality)
{

  struct jpeg_compress_struct cinfo;
  /* This struct represents a JPEG error handler.  It is declared separately
   */
  struct jpeg_error_mgr jerr;
  /* More stuff */
  FILE * outfile;               /* target file */
  JSAMPROW row_pointer[1];      /* pointer to JSAMPLE row[s] */
  int row_stride;               /* physical row width in image buffer */
  
  /* Step 1: allocate and initialize JPEG compression object */

  cinfo.err = jpeg_std_error(&jerr);

  /* Now we can initialize the JPEG compression object. */
  jpeg_create_compress(&cinfo);

  /* Step 2: specify data destination (eg, a file) */
  /* Note: steps 2 and 3 can be done in either order. */

  if ((outfile = fopen(filename, "wb")) == NULL) {
    fprintf(stderr, "can't open %s\n", filename);
    exit(1);
  }
  jpeg_stdio_dest(&cinfo, outfile);

  /* Step 3: set parameters for compression */

  cinfo.image_width = image_width;      /* image width and height, in pixels */
  cinfo.image_height = image_height;
  cinfo.input_components = 1;           /* # of color components per pixel */
  cinfo.in_color_space = JCS_GRAYSCALE; /* colorspace of input image */

  jpeg_set_defaults(&cinfo);

  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  /* Step 4: Start compressor */

  jpeg_start_compress(&cinfo, TRUE);
  
  /* Step 5: while (scan lines remain to be written) */
  /*           jpeg_write_scanlines(...); */

  row_stride = image_width; /* JSAMPLEs per row in image_buffer */

  while (cinfo.next_scanline < cinfo.image_height) {

    row_pointer[0] = & image_buffer[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  /* Step 6: Finish compression */

  jpeg_finish_compress(&cinfo);

  fclose(outfile);

  /* Step 7: release JPEG compression object */

  /* This is an important step since it will release a good deal of memory. */
  jpeg_destroy_compress(&cinfo);
  /* And we're done! */
}

struct my_error_mgr {
  struct jpeg_error_mgr pub;    /* "public" fields */
  
  jmp_buf setjmp_buffer;        /* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

/*
 * Here's the routine that will replace the standard error_exit method:
 */

/*METHODDEF(void)
    my_error_exit (j_common_ptr cinfo)
{*/
  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  /*  my_error_ptr myerr = (my_error_ptr) cinfo->err;*/

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
/*  (*cinfo->err->output_message) (cinfo);*/
  
  /* Return control to the setjmp point */
/*  longjmp(myerr->setjmp_buffer, 1); */
/*}*/

void perr (int i) { fprintf (stderr, "wrong argument %d\n", i); exit (0); };

int main (int argc, char **argv)
{
  int ai=1;
  char *in=0, *out=0;
  unsigned char c1=0, c2=0, *buf;
  FILE *f;
  int w=0, h=0, s=0, q=255;

  if (argc < 4) return 0;
  
  while (argc > ai) {
    if (argv[ai][0] != '-') perr (ai);
    switch (argv[ai][1]) {
    case 'o':
      if (ai >= argc-1) perr (ai);
      out = (char*)malloc (strlen (argv[ai+1]+1));
      if (!out) {
	fprintf (stderr, "cannot store out-file name\n");
	return 0;
      }
      strcpy (out, argv[++ai]);
      break;

    case 'i':
      if (ai >= argc-1) perr (ai);
      in = (char*)malloc (strlen (argv[ai+1]+1));
      if (!in) {
	fprintf (stderr, "cannot store in-file name\n");
	return 0;
      }
      strcpy (in, argv[++ai]);
      break;

    case 'w':
      if (ai >= argc-1) perr (ai);
      w = atoi (argv[++ai]);
      break;

    case 'h':
      if (ai >= argc-1) perr (ai);
      h = atoi (argv[++ai]);
      break;

    case 's': /* scale: 2=0.5, 3=0.333... */
      if (ai >= argc-1) perr (ai);
      s = atoi (argv[++ai]);
      break;

    case 'q': /* quality */
      if (ai >= argc-1) perr (ai);
      q = atoi (argv[++ai]);
      break;

    default:
      perr (ai);
    }
    ai++;
  }

  image_buffer = (JSAMPLE*)malloc (w*h);
  if (!image_buffer) {
    fprintf (stderr, "not enough memory for image buffer (%d * %d => %d octets)\n",
	     w, h, w*h);
    return 0;
  }

  f = fopen (in, "rb");
  if (!f) {
    fprintf (stderr, "cannot open %s: %s\n",
	     in, strerror (errno));
  }

  buf = (unsigned char*)image_buffer;

  image_width = w;
  image_height = h;
  ai = 0;

  while (!feof (f)) {
    c2 = fgetc (f);
    c1 = fgetc (f);
    if (!(ai++%s)) { *buf++ = ((c1>>4)+(c2<<4)); }
  }

  write_JPEG_file (out, q);

  if (in) free (in);
  if (out) free (out);

  return 0;
}




