#! /bin/sh

#
#  Alpha-Mini
#
#  Copyright (C) 2000 Florin Boariu.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
# Semaphore for the CCD cam -- it lives $1 seconds,
# writes on startup $2 in /tmp/cam_semaphore
# and deletes /tmp/cam_semaphore when it dies
#
# Purpose: let the cam wait for several hours but
# keep the server responding
#
# Additionally: right before dieing, it will read $3*$4*2
# bytes from $5 and save them to /tmp/image.raw16
#

TIME="$1"
MESSAGE="$2"
DEVICE="$5"
DEVSTATE="/dev/cam/state"
FILE="/tmp/camsem"

if [ "$TIME"foo = "foo" ];     then rm -f "$FILE"; exit 0; fi
if [ "$MESSAGE"foo = "foo" ];  then rm -f "$FILE"; exit 0; fi
if [ "$DEVICE"foo = "foo" ];   then rm -f "$FILE"; exit 0; fi


if [ ! -r "$FILE" ]; then
    echo No "$FILE"!
    exit 0
fi

echo -n "" > "$FILE"

# wait
while [ "$TIME" -gt "0" ]; do
    if [ ! -r "$FILE" ]; then exit 1; fi
    # someone might have forced a restart

    echo "Out for lunch. I'll be back in $TIME seconds." > "$FILE"
    echo -n "Meanwhile: " >> $FILE
    echo "$MESSAGE" >> "$FILE"
    sleep 1
    TIME=$[TIME-1]
done

# just in case some old image remained undeleted
if [ -x /tmp/image.raw16 ]; then rm /tmp/image.raw16; fi
if [ -x /tmp/image.width ]; then rm /tmp/image.width; fi
if [ -x /tmp/image.height ]; then rm /tmp/image.height; fi

#
# if size and device specified, read the image
#

IMGXSIZE=`cat "$DEVSTATE" | sed -n /^xmax/p | cut -f2 -d=`
IMGYSIZE=`cat "$DEVSTATE" | sed -n /^ymax/p | cut -f2 -d=`

if [ "$IMGXSIZE"foo = "foo" ]; then rm -f "$FILE"; exit 0; fi
if [ "$IMGYSIZE"foo = "foo" ]; then rm -f "$FILE"; exit 0; fi

IMGSIZE=$[$IMGXSIZE * $IMGYSIZE * 2]

if [ ! "$IMGSIZE"foo = "foo" -a -c "$DEVICE" ]; then
    echo "$IMGXSIZE" > /tmp/image.width
    echo "$IMGYSIZE" > /tmp/image.height
    dd if="$DEVICE" of=/tmp/image.raw16 bs="$IMGSIZE" count=1
fi
#else
#    if [ ! "$IMGSIZE"foo = "foo" ]; then echo ONE; fi
#    if [  -c "$DEVICE" ]; then echo TWO; fi
#    echo $IMGSIZE
#    echo $IMGXSIZE
#    echo $IMGYSIZE
#    echo $DEVICE
#fi

rm "$FILE"
