#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <iostream>
#include <string>

#include <sky/core/joblistdir.hh>

#define OPT_SPOOLDIR  'd'
#define OPT_START     's'
#define OPT_LENGTH    'l'
#define OPT_MODULE    'm'
#define OPT_HELP      'h'

void show_help_queue (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS KEY1=VALUE1 KEY2=VALUE2 ...\n\
\n\
Valid OPTIONS are:\n\
\n\
     -d,--spooldir dir  Spool directory to use.\n\
     -s,--start time    Time to start like in \"YYYY/MM/DD hh:mm:ss\".\n\
     -l,--length time   Estimated length in seconds, like in ss.ss.\n\
     -m,--module name   Name of the module able to interpret the job data.\n\
     -h,--help          Print this screen and exits.\n\
\n\
The KEY-VALUE pairs will be passed to the module as job data.\n\
\n", name);
}

int prog_queue (int argc, char **argv)
{
	struct option opts[]={{"spooldir", 1, NULL, OPT_SPOOLDIR},
			      {"start",    1, NULL, OPT_START},
			      {"length",   1, NULL, OPT_LENGTH},
			      {"module",   1, NULL, OPT_MODULE},
			      {"help",     0, NULL, OPT_HELP}};
	int opt;
	string dir, mod;
	Time start=0.0, len=0.0;

	while (0 < (opt=getopt_long(argc, argv, "d:s:l:m:h", opts, NULL))) {
		switch (opt) {
		case OPT_SPOOLDIR:
			dir = optarg;
			break;

		case OPT_START:
			start = to_time(optarg, TIME_FORMAT);
			break;

		case OPT_LENGTH:
			len = atof(optarg);
			break;

		case OPT_MODULE:
			mod = optarg;
			break;

		case OPT_HELP:
			show_help_queue (argv[0]);
			exit(0);
			break;

		default:
			cerr << argv[0] << ": ignoring unknown option "
			     << optarg << endl;
			break;
		}
	}

	JobListDir jobs (dir);
	JobData data;

	printf ("Scheduling: %3.1f %3.1f %s\n", start, len, mod.c_str());

	for (int i=optind; i < argc; i++) {
		string left("");
		char *s = argv[i];
		while (*s && *s!='=') { left+=*s++; };
		data[left] = ++s;
	}

	data.setStart (start);
	data.setLen (len);
	data.setMod (mod);
	if (jobs.schedule(data)) return 0;
	else {
		fprintf (stderr, "Coult not schedule job (not enough time?)\n");
		return 1;
	}
}
