#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <iostream>
#include <string>

#include <link/dlloader.hh>
#include <cronic/joblist.hh>

#define OPT_LIST      'L'
#define OPT_LISTINIT  'i'
#define OPT_SHELL     'S'
#define OPT_START     's'
#define OPT_LENGTH    'l'
#define OPT_MODULE    'm'
#define OPT_HELP      'h'

void show_help_queue (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS KEY1=VALUE1 KEY2=VALUE2 ...\n\
\n\
Valid OPTIONS are:\n\
\n\
     -L,--list lib:sym  Job list module (ibrary and symbol) to use.\n\
     -i,--initlist str  String to initialize job list module\n\
     -S,--shell lib:sym Job shell module (library and symbol) to use\n\
     -s,--start time    Time to start like in \"YYYY/MM/DD hh:mm:ss\".\n\
     -l,--length time   Estimated length in seconds, like in ss.ss.\n\
     -m,--module name   Name of the module able to interpret the job data.\n\
     -h,--help          Print this screen and exits.\n\
\n\
The KEY-VALUE pairs will be passed to the module as job data.\n\
\n", name);
}

int prog_queue (int argc, char **argv)
{
	struct option opts[]={{"list",     1, NULL, OPT_LIST},
			      {"listinit", 1, NULL, OPT_LISTINIT},
			      {"shell",    1, NULL, OPT_SHELL},
			      {"start",    1, NULL, OPT_START},
			      {"length",   1, NULL, OPT_LENGTH},
			      {"module",   1, NULL, OPT_MODULE},
			      {"help",     0, NULL, OPT_HELP}};
	int opt;
	string dir, mod;
	Time start=0.0, len=0.0;
	char lslib[256], *lssym, shlib[256], *shsym;
	string lsinit;
	int i;

	while (0 < (opt=getopt_long(argc, argv, "L:i:S:s:l:m:h", opts, NULL))) {
		switch (opt) {
		case OPT_LIST:
			strncpy (lslib, optarg, 255);
			lslib[255]=0;
			for (i=0; lslib[i]; i++) {
				if (lslib[i]==':') {
					lssym = &(lslib[i+1]);
					lslib[i]=0;
				}
			}
			cout << "Will use list implementation: '" << lssym 
			     << "' from module '" << lslib << "'" << endl;
			break;

		case OPT_LISTINIT:
			lsinit = optarg;
			break;

		case OPT_SHELL:
			strncpy (shlib, optarg, 255);
			shlib[255]=0;
			for (i=0; shlib[i]; i++)
				if (shlib[i]=':') {
					shsym = &shlib[i+1];
					shlib[i]=0;
				}
			cout << "Will use shell implementation: '" << shsym
			     << "' from module '" << shlib << "'" << endl;
			break;

		case OPT_START:
			start = to_time(optarg, TIME_FORMAT);
			break;

		case OPT_LENGTH:
			len = atof(optarg);
			break;

		case OPT_MODULE:
			mod = optarg;
			break;

		case OPT_HELP:
			show_help_queue (argv[0]);
			exit(0);
			break;

		default:
			cerr << argv[0] << ": invalid option "
			     << optarg << endl;
			return 0;
		}
	}

	try {
		DLLoader dl;
		JobList *jobs = (JobList *)dl.create (lslib, lssym);
		if (!jobs) {
			cerr << "Cannot get job list object \""<< lssym
			     << "\" from \"" << lslib << "\"" << endl;
			return 0;
		}
		jobs->init (lsinit);
		JobData data;
		
		// key=value (parsing job data)
		for (int i=optind; i < argc; i++) {
			string left("");
			char *s = argv[i];
			while (*s && *s!='=') { left+=*s++; };
			data[left] = ++s;
		}
		
		data.setStart (start);
		data.setLen (len);
		data.setMod (mod);
		if (jobs->schedule(data)) return 0;
		else {
			cerr << "Coult not schedule job (not enough time?)" << endl;
			return 1;
		}
		
		jobs->cleanup();
	} catch (exception &e) {
		cerr << endl << "\t Fatal error: " << e.what() << endl << endl;
	}
}
