#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <iostream>
#include <string>

#include <link/dlloader.hh>

#include <cronic/loop.hh>
#include <cronic/joblist.hh>
#include <cronic/jobshell.hh>

#include <cfg/cfg.hh>

#define OPT_CONFIG    'C'
#define OPT_LIST      'L'
#define OPT_LISTINIT  'l'
#define OPT_SHELL     'S'
#define OPT_SHELLINIT 's'
#define OPT_TIMESLICE 't'
#define OPT_HELP      'h'

void show_help_run (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS\n\
\n\
Valid OPTIONS are:\n\
\n\
     -C,--config file   Load configuration from file 'file'\n\
     -L,--list lib:sym  Job list module (library and symbol) to use.\n\
     -l,--initlist str  String to initialize job list module\n\
     -S,--shell lib:sym Job shell module (library and symbol) to use\n\
     -s,--shellinit str String to initialize job shell module\n\
     -t,--timeslice sec The timeslices to use (in seconds).
     -h,--help          Print this screen and exits.\n\
\n", name);
}

int prog_run (int argc, char **argv)
{
	struct option opts[]={{"config",   1, NULL, OPT_LIST},
			      {"list",     1, NULL, OPT_LIST},
			      {"listinit", 1, NULL, OPT_LISTINIT},
			      {"shell",    1, NULL, OPT_SHELL},
			      {"shellinit",1, NULL, OPT_SHELLINIT},
			      {"timeslice",1, NULL, OPT_TIMESLICE},
			      {"help",     0, NULL, OPT_HELP}};
	int opt;
	string dir;
	double slice=1.0;
	char lslib[256], *lssym, shlib[256], *shsym;
	map<string,string> shcfg, lscfg;
	int i;
	Config config;

	while (0 < (opt=getopt_long(argc, argv, "C:L:l:S:s:t:h", opts, NULL))) {
		switch (opt) {
		case OPT_CONFIG:
			config.load (optarg);
			break;

		case OPT_LIST:
		  strncpy (lslib, optarg, 255);
		  lslib[255]=0;
		  for (i=0; lslib[i]; i++)
			  if (lslib[i]==':') {
				  lssym = &lslib[i+1];
				  lslib[i]=0;
			  }
		  cout << "Will use list implementation: '" << lssym 
		       << "' from module '" << lslib << "'" << endl;
		  config.val("modules", "list lib") = lslib;
		  config.val("modules", "list sym") = lssym;
		  break;

		case OPT_LISTINIT:
			strToMap (optarg, config.sec("list init"));
			break;

		case OPT_SHELL:
			strncpy (shlib, optarg, 255);
			shlib[255]=0;
			for (i=0; shlib[i]; i++)
				if (shlib[i]==':') {
					shsym = &shlib[i+1];
					shlib[i]=0;
				}
			cout << "Will use shell implementation: '" << shsym 
			     << "' from module '" << shlib << "'" << endl;
			config.val("modules", "shell lib") = shlib;
			config.val("modules", "shell sym") = shsym;
			break;

		case OPT_SHELLINIT:
			strToMap (optarg, config.sec("shell init"));
			break;

		case OPT_TIMESLICE:
			slice = atof(optarg);
			break;

		case OPT_HELP:
			show_help_run (argv[0]);
			exit(0);
			break;

		default:      
			return 0;
		}
	}

	try {
		DLLoader dl;
		config.val("modules", "list lib") = lslib;
		config.val("modules", "list sym") = lssym;
		JobList *jobs = (JobList *)dl.create (config.val("modules", "list lib"),
						      config.val("modules", "list sym"));
		if (!jobs) {
			cerr << "Cannot get job list object \""<< lssym
			     << "\" from \"" << lslib << "\""<< endl;
			return 0;
		}
		jobs->init (config.sec("list init"));
		JobShell *shell = (JobShell *)dl.create (config.val("modules", "shell lib"),
							 config.val("modules", "shell sym"));
		shell->init (config.sec("shell init"));
		if (!shell) {
			cerr << "Cannot get shell object \""<< shsym
			     << "\" from \"" << shlib << "\""<< endl;
			return 0;
		}	
		
		Loop (slice, jobs, shell).run();
	} catch (exception &e) {
		cerr << endl << "\tFatal error: " << e.what() << endl << endl;
	}
}
