#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>

#include <drivemsg.h>

#define OPT_DAEMON 'd' /* start in daemon mode */
#define OPT_MSGKEY 'k' /* message queue key */
#define OPT_HELP   'h' /* show help */

void show_help_stickd (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS\n\
\n\
Valid OPTIONS are:\n\
\n\
     -d,--daemon        Fork into background (daemon mode)\n\
     -k,--msgkey key    Message queue key\n\
     -h,--help          Shows this screen.\n\
\n\
Joystick Control Daemon.\n\
\n", name);
}

struct teleinfo {
	int azmfd, elvfd;
	double clx, cly, clz;
	double corazm, corelv; // azimuth and elevation correction
	int qid;
};

int main (int argc, char **argv)
{
	struct option opts[]={{"daemon",   0, NULL, OPT_DAEMON},
			      {"msgkey",   1, NULL, OPT_MSGKEY},
			      {"help",   0, NULL, OPT_HELP}};
	char daemon=0;
	int opt;
	const char *tmp;
	int key=0, qid;
	struct message msg;
	char run=1;
	char c;
	char d[100];

	char nr[10];

	while (0 < (opt=getopt_long(argc, argv, "a:c:de:hk:", opts, NULL))) {
		switch (opt) {
		case OPT_DAEMON:
			daemon = 1;
			break;
			
		case OPT_MSGKEY:
			key = atoi(optarg);
			break;

		case OPT_HELP:
			show_help_stickd (argv[0]);
			exit(0);
			break;

		default:
			break;
		}
	}

	if (daemon) switch (fork()) {
	case -1:
		perror ("fork");
		return -1;

	case 0:
		printf ("drvctl forking into background\n");
		return 0;

	default:
		break;
	}

	if (-1 == (qid = msgget (key, 0))) {
		perror ("msgget");
		return 0;
	}

	printf ("queue id: %d\nqueue key: %d\n", qid, key);

 loop:
	do {
		bzero (&msg, sizeof (struct message));
		msg.mtype = MSG_TELE;

		printf ("need: <cmd> <time> <azm> <elv> <dst>\n");

		scanf ("%x", &(msg.cmd.cmd));
		scanf ("%s", d);
		msg.cmd.data[D_TIME] = atof(d);
		scanf ("%s", d);
		msg.cmd.data[D_AZM] = atof(d);
		scanf ("%s", d);
		msg.cmd.data[D_ELV] = atof(d);
		scanf ("%s", d);
		msg.cmd.data[D_DST] = atof(d);

		printf ("Thank you.\n");
		
		if (-1 != msgsnd (qid, (struct msgbuf*)&msg,
				  sizeof (struct telecmd), 0)) {
			printf ("sent: %d with %f %f %f %f\n", msg.cmd.cmd,
				msg.cmd.data[0],msg.cmd.data[1],msg.cmd.data[2],msg.cmd.data[3]);
		} else {
			perror ("msgsnd");
			run = 0;
		}
	} while (run);

	if (-1 == (msgctl (qid, IPC_RMID, NULL))) {
		perror ("msgctl(IPC_RMID)");
		return 0;
	}

	return 0;
}

struct teleinfo *tele_open (const char *azm, const char *elv)
{
	struct teleinfo *tele = (struct teleinfo*)malloc (sizeof(struct teleinfo));
	
	if (!tele) return NULL;

	if (0 < (tele->azmfd = open (azm, O_RDWR))) {
		perror (azm);
		exit (-1);
	}

	if (0 < (tele->azmfd = open (azm, O_RDWR))) {
		perror (azm);
		exit (-1);
	}

	return tele;
}

void tele_close (struct teleinfo *tele)
{
	close (tele->azmfd);
	close (tele->elvfd);
	free (tele);
}
