#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>

#include <sky/drv/joblistdrv.hh>
#include <sky/drv/jobshelldrv.hh>
#include <cronic/loop.hh>
#include <tcpip/server.hh>

#include <cfg/cfg.hh>

#define OPT_DAEMON 'd'
#define OPT_SLICE  's'
#define OPT_HELP   'h'

void show_help_drvctl (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS\n\
\n\
Valid OPTIONS are:\n\
\n\
     -h,--help          Shows this screen.\n\
\n\
yadda yadda
\n", name);
}

int drvctl (void);

Config config;

int main (int argc, char **argv)
{
	struct option opts[]={{"help",   0, NULL, OPT_HELP}};
	int opt;
	double slice;

	while (0 < (opt=getopt_long(argc, argv, "h", opts, NULL))) {
		switch (opt) {
		case OPT_HELP:
			show_help_drvctl (argv[0]);
			exit(0);
			break;

		default:
			return 0;
			break;
		}
	}

	cerr << "loading config file..." << endl;
	config.load ("/etc/drvctl.conf");		

	cerr << "starting server..." << endl;

	return drvctl();
}

/*
 * in an infinite-loop matter do:
 *    o listen for a new client
 *    o get the list layout data
 *    o init list
 *    o get the coordinate stream untile end-of-tramsmission
 *    o close connection and listen for the next client
 *
 * the data is split in two parts: the header information
 * and the numbers. they are separated by an empty line.
 * the numbers arrive in tuples, one tuple each line.
 *
 * the header is a key/value pair block;
 * valid keys are:
 * 
 * hello:     the hello string (with version information, whatever...)
 * timebase:  the time base.
 * tpsize:    the size of the tuples. first value is always time (added to the timebase)
 *            following values are coordinates. p.s.: not needed. size can be guessed
 *            from the tptype parameter.
 * tpno:      number of the tuples (a.k.a. ring buffer size).
 *            we will read tuples as long as they come, regardless of what
 *            value 'tpno' is. the stop signal an empty line again (end-of-transmission).
 *            setting tpno to a small value (5 for example) and sending data just-in-time
 *            will result into a streaming effect for coordinates.
 * ringsize:  same as tpno, overrules tpno
 * tptype:    tuple data type. one of "axis speed", "cart position", "altazm position"
 *            ("elvazm" is same as "altazm")
 */
int drvctl (void)
{
	try {

		TCPServer serv (23231, 1);
		
		serv.start();
		
		while (1) {
			char buf[256];
			int rb;
			int confd = serv.accept();

			// get the hello from the client
			ifstream is (confd);
			string head;
			char line[256];
			while (!(is.getline(line,255)).eof() && line[0] && line[0]>=32) {
				cout << "got " << strlen(line) << " bytes: " << line << endl;
				cout << (int)line[0] << endl;
				line[255]=0;
				head += line;
			}			

			map<string,string> ccfg, lcfg, scfg;
			strToMap (head, ccfg);

			cout << "starting config" << endl; cout.flush();

			bool speed = ccfg["tptype"] == "axis speed";

			// list configuration
			lcfg["timebase"] = ccfg["timebase"];
			lcfg["jobs"]     = ccfg["tpno"];
			lcfg["jobs"]     = ccfg["ringsize"];
			lcfg["fast"]     = "yes";
			lcfg["semkey"]   = config.val(speed ? "speed list" : "pos list", "semkey");
			lcfg["shfile"]   = config.val(speed ? "speed list" : "pos list", "shfile");
			lcfg["newshf"]   = "yes";
			lcfg["cleanup"]  = "no";

			cout << "init list" << endl; cout.flush();

			// init list
			JobListDrv joblist;
			joblist.init(lcfg);
			cout << "locking list" << endl; cout.flush();

			cout << "recv data" << endl; cout.flush();

			// receive the data
			JobData d;
			int i;
			string &time = d["time"];
			string &x    = d["x"];
			string &y    = d["y"];
			string &z    = d["z"];
			while (!(is.getline(line,255)).eof() && line[0] && line[0] != 13) {
				time.erase();
				x.erase();
				y.erase();
				z.erase();
				i = 0;
				while (line[i]!=0 && line[i]!=' ') { time+=line[i++]; }
				if (line[i]) i++;
				while (line[i]!=0 && line[i]!=' ') { x+=line[i++]; }
				if (line[i]) i++;
				while (line[i]!=0 && line[i]!=' ') { y+=line[i++]; }
				if (line[i]) i++;
				while (line[i]!=0 && line[i]!=' ') { z+=line[i++]; }
				if (line[i]) i++;
				cout << "scheduling: " << time << " "
				     << x << " " << y << " " << z << endl;
				joblist.lock();
				joblist.schedule (d);
				joblist.unlock();
			}

			// the loop will be executed in an external application,
			// now that we have semaphore-locked lists :)
			// see crtools for details (crrun, to be more precise)
		}
		
		// we'll never reach this
		serv.stop();
		
		return 0;

	} catch (exception &e) {
		cerr << "Fatal: "; cerr.flush();
		cerr << e.what() << endl;
		return -1;
	}
}







