#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <iostream>
#include <string>

#include <linux/stepeng_ioctl.h>
#include <sys/ioctl.h>

#define OPT_DAEMON 'd'
#define OPT_SLICE  's'
#define OPT_HELP   'h'

void show_help_estopd (const char *name)
{
	fprintf (stderr, "\
Usage: %s OPTIONS DEVICE\n\
\n\
Valid OPTIONS are:\n\
\n\
     -d,--daemon        Fork to background after start (daemon mode).\n\
     -s,--slice time    Time slice (in seconds).\n\
     -h,--help          Shows this screen.\n\
\n\
DEVICE specifies the engine device to watch.
\n", name);
}

void estopd_watch (const char *dev);

int main (int argc, char **argv)
{
	struct option opts[]={{"daemon mode", 0, NULL, OPT_DAEMON},
			      {"slice",  1, NULL, OPT_SLICE},
			      {"help",   0, NULL, OPT_HELP}};
	int opt;
	double slice;
	bool daemon=true;

	while (0 < (opt=getopt_long(argc, argv, "ds:h", opts, NULL))) {
		switch (opt) {
		case OPT_DAEMON:
			daemon = true;
			break;

		case OPT_SLICE:
			slice = atof(optarg);
			break;

		case OPT_HELP:
			show_help_queue (argv[0]);
			exit(0);
			break;

		default:
			cerr << argv[0] << ": ignoring unknown option "
			     << optarg << endl;
			break;
		}
	}

	if (daemon) switch (fork()) {
	case -1:
		perror ("fork");
		return -1;

	case 0:
		printf ("estopd forking into background\n");
		return 0;

	default:
		estopd_watch (argv[0]);
		break;
	}

	return 0;
}

void estopd_watch (const char *dev)
{
	int fd = open (dev, O_RDWR);
	
	if (fd < 0) {
		perror (dev);
		exit (-1);
	}
	
	unsigned char lims;
	double stop=0.0;
	
	while (1) {
		ioctl (fd, HPE_CHKLIMITS, &lims);
		if (HPE_HILIM(lims)) {
			printf ("%s: high limit\n", dev);
			ioctl (fd, HPE_TOSPEED, &stop);
		}
		if (HPE_LOLIM(lims)) {
			printf ("%s: low limit\n", dev);
			ioctl (fd, HPE_TOSPEED, &stop);
		}
	}
}
