#include <sky/user/cgi.hh>
#include <sky/core/joblistdir.hh>
#include <sky/user/user.hh>
#include <sky/user/cgi2.hh>
#include <fstream>
#include <map>
#include <string>

struct cgi_opt {
	char *key;
	char *val;
};

#define OPT_MIN     0
#define IMG_TOP     0
#define IMG_LEFT    1
#define IMG_WIDTH   2
#define IMG_HEIGHT  3
#define IMG_HBIN    4
#define IMG_VBIN    5
#define IMG_ITIME   6
#define OBJ_DESC    7
#define OBJ_ROUTE   8
#define USR_NAME    9
#define USR_PASS   10
#define JOB_START  11
#define JOB_DESC   12
#define OPT_MAX    12

#define INIT_TIME 60.0     // 60 seconds for init. ONLY TESTING!
#define CLEANUP_TIME 60.0  // cleanup time

#define JOBDIR "/home/florin/pro/skylark/test"
//#define JOBDIR "."

int main (int argc, char **argv)
{
	cgi_init();
	printf("Content-type: text/html\n\n");
	
	char *jobopts[] = {"itop", "ileft", "iwidth", "iheight", "ihbin", 
			   "ivbin", "iitime", "odesc", "oroute", "uname",
			   "upass", "jstart", "jdesc", NULL};

	const char *sesid = cgi_param("sesid");
	if (!sesid) sesid="";
	map<string,string> chmap;

	UserSession ses(sesid);
	ses.restore();
	chmap["!!SESID!!"] = ses.id();
	
	string uname = ses.getVal ("uname", "");
	string upass = ses.getVal ("upass", "");
	string skin = ses.getVal ("skin", "default");

	chmap["!!UNAME!!"] = uname;
	chmap["!!UPASS!!"] = upass;

	if (cgi_param("showform")) {
		cgi_send_file (skin+"/insjob/jobform.tpl", &chmap);
		return 0;
	}

	if (!skyCheckUser(uname.c_str(), upass.c_str())) {
		chmap["!!ERROR!!"] = "You don't have sufficient permission.";
		cgi_send_file (skin+"/etc/err.tpl", &chmap);	       
		return 0;
	}

	const char *tmp = cgi_param("jstart");
	Time tstart = tmp ? to_time(tmp, TIME_FORMAT) : 0.0;
	if (tstart < UTC()) tstart = UTC()+100.0;
	//printf ("time: %2.2f, utc: %2.2f\n", tstart, UTC());
	tstart -= INIT_TIME;
	tmp = cgi_param ("itime");
	Time tlength = tmp ? atof(tmp) : 0.0 +INIT_TIME+CLEANUP_TIME+10.0; // 10 paranoia seconds

	// if we have a valid job id, we will try to replace the given
	// job with the new one
	tmp = cgi_param("jobid");
	string jobid = tmp ? tmp : "";
	for (int i=0; i < jobid.length(); i++) if (jobid[i]==' ') jobid[i]='+';


	JobListDir jobs;
	JobData data;
	jobs.init (JOBDIR);
	jobs.lock();
	//cerr << "jobid: " << jobid << endl;
	if (jobid.length()) {
		//cout << "attempt to delete " << jobid << endl;
		JobData old;
		if (jobs.find(jobid, old)) {
			jobs.validate(old);
			// check if user == job_user, then rm job
			if (old.fetch("uname") != uname) {
				chmap["!!ERROR!!"] = "This is not your job!";
				cgi_send_file (skin+"/etc/err.tpl", &chmap);
				return 0;
			} else {
				jobs.discard (old.id());
				//cout << "deleted " << old.id() << endl;
			}
		} else {
			chmap["!!ERROR!!"] = "The specified job ID is not valid!";
			cgi_send_file (skin+"/etc/err.tpl", &chmap);
			return 0;
		}
	}

	//cout << "inserting new job" << endl;

	data.setStart(tstart);
	data.setLen(tlength);
	data.setMod("skycore");

	for (int i = 0; jobopts[i]; i++) {
		//printf ("asking for %s\n", jobopts[i]); fflush (stdout);
		const char *str = cgi_param (jobopts[i]);
		if (str) data[jobopts[i]] = str;
	}
	data["uname"] = uname;
	data["upass"] = upass;
	bool ret = jobs.schedule(data);
	jobs.unlock();

	chmap["!!TITLE!!"] = "Results";
	chmap["!!HEAD!!"] = ret ? "Succes" : "Failed";
	chmap["!!MESSAGE!!"] = ret ? "The job was successfully scheduled."
		: "The job has not been scheduled. Probably not enough time?";
	cgi_send_file (skin+"/etc/msg.tpl", &chmap);

	cgi_done();
}





