#include <sky/user/cgi.hh>
#include <sky/core/joblistdir.hh>
#include <sky/user/cgi2.hh>
#include <time.h>

#define JOBDIR "/home/florin/pro/skylark/test"

int main (int argc, char **argv)
{
	cgi_init();
	printf("Content-type: text/html\n\n");

	const char *sesid  = cgi_param("sesid");
	if (!sesid) sesid = "";
	UserSession ses(sesid);

	// templates:
	// pghead.tpl, jobent.tpl, pgtail.tpl (files)

	// colors:
	// jobbg1 jobbg2 jobfg1 jobfg2 (cgi-vars)

	// tpl-variables:
	// !!SESID!! -- session id

	// !!JOBBG!! -- job background color
	// !!JOBFG!! -- job foreground color
	// !!JOBSTART!!  -- job start time
	// !!JOBLENGTH!! -- job length
	// !!JOBOWNER!!  -- the owner of the job (user name)
	// !!JOBDESC!!   -- an owner's description of the job
	// !!JOBID!!     -- the ID of the job

	JobListDir jobs(JOBDIR);
	JobData data;

	map<string,string> tpl;

	tpl["!!SESID!!"] = ses.id();
	string skin = ses.getVal("skin", "default");
	cgi_send_file ((skin+"/lsjob/pghead.tpl"), &tpl);
	string jobent = (skin+"/lsjob/jobent.tpl");
	const char *bgcols[2], *fgcols[2];

	// load (default) colors
	bgcols[0] = ses.getVal ("jobbg1", "#c0c0c0").c_str();
	bgcols[1] = ses.getVal ("jobbg2", "#ffffff").c_str();
	fgcols[0] = ses.getVal ("jobfg1", "#000000").c_str();
	fgcols[1] = ses.getVal ("jobfg2", "#000000").c_str();

	int colid=0;

	jobs.lock();

	if (jobs.firstItem(data)) {
		do {
			jobs.validate (data);

			tpl["!!JOBBG!!"] = bgcols[colid];
			tpl["!!JOBFG!!"] = fgcols[colid];

			char tmp[51];
			//snprintf (tmp, 50, "%.1f", data.start());
			time_t t = time_t (data.start());
			tpl["!!JOBSTART!!"] = ctime(&t);

			snprintf (tmp, 50, "%.1f", data.len());
			tpl["!!JOBLENGTH!!"] = tmp;

			tpl["!!JOBOWNER!!"] = data.fetch("uname");
			tpl["!!JOBDESC!!"] = data.fetch("jdesc");
			tpl["!!JOBID!!"] = data.id();

			if (!cgi_send_file (jobent, &tpl)) {
				cerr << "cannot find template " << jobent << endl;
			}

			colid = !colid;
		} while (jobs.nextItem(data));
	}

	jobs.unlock();

	cgi_send_file ((skin+"/lsjob/pgtail.tpl"), &tpl);

	cgi_done();
}





