#include <tcpip/client.hh>

#include <sys/types.h>
#include <sys/socket.h>

#include <stdexcept>
#include <linux/joystick.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>

inline int recv_pos (int fd, double *x1, double *x2)
{
	JS_DATA_TYPE jd;
	if (-1 == read (fd, &jd, sizeof (JS_DATA_TYPE))) {
		return -1;
	}
	
	*x1 = jd.x;
	*x2 = jd.y;
}

int main (int argc, char **argv)
{
	int jfd = open (argv[1], O_RDONLY);
	char data[100], init[]=
	  "timebase=0.0\n"
	  "hello=Hello\n"
	  "tpsize=1\n"
	  "tptype=\"axis speed\"\n\n";
	JS_DATA_TYPE js, jsz;

	if (-1 == jfd) {
		cerr << "unable to open joystick device" << endl;
	}

	if (-1 == read (jfd, &(jsz), sizeof (JS_DATA_TYPE))) {
		perror ("readj");
		exit (-1);
	}

	try {
		TCPClient cli;
	  
		cli.connect (argv[1], atoi(argv[2]));
		send (cli.fd(), &init, strlen(init), 0);

		double x1, x2;
		int oldx, oldy;
		while (-1 != read (jfd, &js, sizeof(JS_DATA_TYPE))) {
			usleep (100000);
			int x = js.x-jsz.x;
			int y = js.y-jsz.y;		       
			
			if (x != oldx || y != oldy) {
				sprintf (data, "0.0 %f %f 0\n",
					 double(x)*2.2e-5, double(y)*2.2e-5);
				printf ("sending %s", data);
				send (cli.fd(), data, strlen(data), 0);
			}
			oldx = x;
			oldy = y;
			if (js.buttons) goto done;
		}
	done:
		cli.disconnect();

	} catch (exception &e) {
		cerr << "caught: " << e.what() << endl;
	}

	close (jfd);
}
