#include <sky/user/cgi.hh>
#include <sky/user/cgi2.hh>
#include <sky/user/user.hh>
#include <sky/core/joblistdir.hh>

#include <map>

#define JOBDIR "/home/florin/pro/skylark/test"

struct app {
	map<string,string> chmap;
	string skin, rdonly;
	UserSession ses;
};

// init cgi
// check presence of jobid
// load session data,
bool app_init (struct app *app)
{
	cgi_init();
	cout << "Content-type: text/html" << endl << endl;

	const char *sesid = cgi_param("sesid");
	if (!sesid) sesid="";

	app->ses = UserSession (sesid);
	app->ses.restore();

	app->chmap["!!SESID!!"] = app->ses.id();
	app->chmap["!!USER!!"]  = app->ses.getVal("uname");	

	app->skin = app->ses.getVal("skin", "default");
	
	const char *tmp = cgi_param("rdonly");
	app->rdonly = tmp ? tmp : "yes";
}

bool get_jobid (struct app *app, string &jobid)
{
	const char *_jobid = cgi_param("jobid");
	if (!_jobid) return false;

	// change SPACES to PLUS
	jobid.erase();
	for (int i=0; _jobid[i]!=0; i++) jobid += _jobid[i]==' ' ? '+' : _jobid[i];
}

bool get_pghead (struct app *app, string &name)
{
	if (app->rdonly == "yes")
		name = app->skin+"/viewjob/rohead.tpl";
	else 
		name = app->skin+"/viewjob/rwhead.tpl";
	return true;
}

bool get_pgtail (struct app *app, string &name)
{
	if (app->rdonly == "yes")
		name = app->skin+"/viewjob/rotail.tpl";
	else 
		name = app->skin+"/viewjob/rwtail.tpl";
	return true;
}

bool get_pgjobopt (struct app *app, string &name)
{
	if (app->rdonly == "yes")
		name = app->skin+"/viewjob/rojobopt.tpl";
	else 
		name = app->skin+"/viewjob/rwjobopt.tpl";
	return true;
}

void app_error (struct app *app, const string &error)
{
	app->chmap["!!ERROR!!"] = error;
	cgi_send_file (app->skin+"/etc/err.tpl", &(app->chmap));
}

void app_close (struct app *app)
{
	app->ses.store();
	cgi_done();
}

int main (int argc, char **argv)
{
	struct app app;
	string jobid;
	
	if (!app_init(&app)) return 0;
	
	if (!get_jobid(&app, jobid)) {
		app_error (&app, "No job ID specified.");
		return 0;
	}
	app.chmap["!!JOBID!!"] = jobid;
		
	if (!skyCheckUser(app.ses.getVal("uname").c_str(), app.ses.getVal("upass").c_str())) {
		app_error (&app, "Permission denied. Have you correctly logged in?");
		return 0;
	}

	// load job data
	JobListDir jobs(JOBDIR);
	JobData data;
	jobs.lock();       
	if (!jobs.find (jobid.c_str(), data)) {
		app_error (&app, "Could not find the specified job.");
		return 0;
	}
	jobs.validate(data);

	// check whether this is our job
	if (data.fetch("uname") != app.ses.getVal("uname")) {
		app_error (&app, "Permission denied: you're not job owner!");
		return 0;
	}

	string pghead, pgtail, pgjobopt;
	get_pghead (&app, pghead);
	get_pgtail (&app, pgtail);
	get_pgjobopt (&app, pgjobopt);

	cgi_send_file (pghead, &(app.chmap));

	map<string,string> transl;
	transl["itop"] = "Image Top";
	transl["ileft"] = "Image Left";
	transl["iwidth"] = "Image Width";
	transl["iheight"] = "Image Height";
	transl["ihbin"] = "Horizonal Binning";
	transl["ivbin"] = "Vertical Binning";
	transl["iitime"] = "Integration Time";
	transl["odesc"] = "Object Description";
	transl["oroute"] = "Object Route";
	transl["uname"] = "Job Owner";
	transl["uemail"] = "Owner's Address";
	transl["jstart"] = "Start Time";

	// output
	for (map<string,string>::iterator i = transl.begin();
	     i != transl.end(); i++) {
		if ((*i).first != "upass") { // don't show password
			app.chmap["!!OPTKEY!!"] = (*i).first;
			app.chmap["!!OPTDESC!!"] = (*i).second;
			app.chmap["!!OPTDATA!!"] = data.fetch((*i).first);
			cgi_send_file (pgjobopt, &(app.chmap));
		}
	}

	jobs.unlock();

	cgi_send_file (pgtail, &(app.chmap));

	app_close (&app);
}





