#ifndef __STEPENG_IOCTL_H
#define __STEPENG_IOCTL_H

#ifndef HPE_MAJOR
#define HPE_MAJOR 111
#endif

/*
 * ioctl numbers
 *
 * for the whole driver goes:
 *   o between user and kernel, speed is measured in
 *     gradients/second
 *   o mileage is given in gradients
 *   o time is measured in seconds
 *   o all values are double precision floats (doubles)
 */

#define HPE_HILIM(lim)  (!(lim&0x80))
#define HPE_LOLIM(lim)  (!(lim&0x40))

struct hpe_engine_state {
	double speed;      /* the current device speed */
	double time;       /* the device time (a.k.a. when was the state read) */
	double pos;        /* the position at the given time */
	signed char spdsign, possign; /* if -1 sign is negative */
	unsigned char lim; /* the limits */
};

#define IOCTL(a,b,c) ((a<<8)+b)

/* engine control (movement) */
#define HPE_MOVEBYGRAD    1110  /* move by the given gradients */
#define HPE_TOSPEED       1111  /* try to reach the given speed */

#define HPE_SETSPEED      1112  /* set the speed, no smoothing! */
#define HPE_GETSPEED      1113

#define HPE_MEASURESPEED  1114  /* measure the speed */
#define HPE_MILEAGE       1115

#define HPE_CHKLIMITS     1116 /* check the limit bits (hi, lo) */

#define HPE_GETSTATE      1117    /* get the whole engine state */
#define HPE_RENEWSTATE    1118    /* like the above, but recompute state */

/* timer commands */
#define HPE_GETCLOCK      11110
#define HPE_WAIT          11111

/* engine dependencies (transformation constants rad => data) */
/*  #define HPE_SETUSEACCEL   11120, double*) */ /* the acceleration to be used */
/*  #define HPE_GETUSEACCEL   11121, double*) */
/*  #define HPE_SETMAXACCEL   11122, double*) */ /* the maximum acceleration */
/*  #define HPE_GETMAXACCEL   11123, double*) */
/*  #define HPE_SETSPEEDSLICE 11124, double*) */ /* speed slice */
/*  #define HPE_GETSPEEDSLICE 11125, double*) */ /* speed slice */
#define HPE_SETPOSFAC     11126 /* translation factors for speed and position */
#define HPE_GETPOSFAC     11127
#define HPE_SETSPDFAC     11128
#define HPE_GETSPDFAC     11129

#define HPE_SETRAMP      11140 /* get and set acceleration ramp */
#define HPE_GETRAMP      11141


/* controller commands */
#define HPE_RESET         11130  /* reset the PIC controler */

/* 
 * new concept: application specifies acceleration ramp 
 * this way the driver does not have to bother with hardware restrictions
 * like maximum speed, maximum acceleration, different accel behaviour
 * for different initial speeds... it just gets an accel ramp and uses it.
 * the application is responsible for providing valid ramp data;
 */

/* acceleration ramp stuff */

struct rampitem {
	double speed;
	double dist;
};

#define HPE_RAMP_MAX 100
struct hpe_accel_ramp {
	int size;
	double tslice;
	/*
	 * set the arrays at the end. this way we have at least one small chance
	 * to remain binary compatible when HPE_RAMP_MAX was changed
	 */ 
	struct rampitem ramp[HPE_RAMP_MAX];
};

#endif /* __STEPENG_IOCTL_H */




