#define __BUILDING_HPE__

#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/sys.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/config.h>
#include <asm/unistd.h>

#include <linux/fs.h>
#include <linux/wrapper.h>
#include <linux/malloc.h>
#include <linux/init.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/segment.h>
/*#include <math.h>*/
#include <linux/string.h>
#include <linux/module.h>

#include <linux/stepeng.h>
#include <linux/stepeng_ioctl.h>

struct hpe_devinfo hpe_dev[HPE_MAX_DEV];

static devfs_handle_t devfs_handle = NULL;

static int HPEMajor;           /* the device's major number */
unsigned int  HPEPort  =0x388; /* the base port address of the engine microcontroler */
unsigned char HPEconIO =0x00;  /* contents of the i/o register */
unsigned char HPEconAux=0x00;  /* contents of the aux register */
unsigned char HPEconDac=0x00;  /* contents of the dac register */
double HPEDefSpdFactor=874963.552725; /* the default factor rad=>dev - speed */
double HPEDefPosFactor=2037.18327156; /* the default factor rad=>dev - position */
/* struct hpe_accel_ramp accel_ramp; *//* the acceleration ramp */


/*
 * attempt to open
 */
static int hpe_open (struct inode *inode, struct file *file)
{
	int min = MINOR(file->f_dentry->d_inode->i_rdev);

	printk (HPE_NAME": open\n");

	/*
	 * minor numbers: engine0-ctrl, engine0-stat, engine1-ctrl, engine1-stat...
	 * ctrl-devices: read/write, exclusively
	 * stat-devices: read-only, used by several applications
	 */

	if (min%2 && hpe_dev[min/2].open)
		return -EBUSY;

	hpe_dev[min/2].open++;

#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif

	return SUCCESS;
}

/*
 * close the device
 */
static int hpe_release (struct inode *inode, struct file *file)
{
	int min = MINOR(file->f_dentry->d_inode->i_rdev);
	printk (HPE_NAME": release\n");

	hpe_dev[min/2].open--;

#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif

	return SUCCESS;
}

/*
 * read from the dev
 */
static ssize_t hpe_read (struct file *file,
			 char *buffer, 
			 size_t len,
			 loff_t offset)
{
	int rb=0;

	/*printk (HPE_NAME": read %d octets\n", rb);*/
	return rb;
}

/*
 * write to the dev
 */
static ssize_t hpe_write (struct file *file)
{
	int wb=0;

	printk (HPE_NAME": wrote %d octets\n", wb);
	return wb;
}

/*
 * the ioctl swiss army knife ;)
 */
static int hpe_ioctl (struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long param)
{
	int ret=SUCCESS;
	double data;
	uint8_t minor = MINOR(file->f_dentry->d_inode->i_rdev);
	uint8_t devnr = (int)((double)minor/2.0);

#ifdef DEBUG
	printk (HPE_NAME": got cmd %d for engine %d\n", cmd, devnr);
#endif

	switch (cmd) {
	case HPE_MOVEBYGRAD:
		printk (HPE_NAME": ioctl movebygrad: not yet implemented\n");
		break;

	case HPE_TOSPEED:
		if (minor%2) return -EIO; /* read-only for state devices */
		copy_from_user (&data, (double *)param, sizeof(double));		
		data = hpe_goto_speed (devnr, data);
		copy_to_user ((double*)param, &data, sizeof(double));
		break;

	case HPE_SETSPEED:
		if (minor%2) return -EIO;
		copy_from_user (&data, (double *)param, sizeof(double));
		hpe_set_speed (devnr, data);
		break;

	case HPE_GETSPEED:
		copy_to_user ((double *)param, &(hpe_dev[devnr].estate.speed),
			      sizeof(double));
		break;

	case HPE_MEASURESPEED:
		printk (HPE_NAME": ioctl measurespeed: not yet implemented\n");
		break;

	case HPE_MILEAGE:
		hpe_set_speed(devnr, hpe_dev[devnr].estate.speed);
		copy_to_user((double*)param, &(hpe_dev[devnr].estate.pos),
			     sizeof(double));
			   
		break;

	case HPE_CHKLIMITS:
		hpe_set_speed(devnr, hpe_dev[devnr].estate.speed);
		copy_to_user((unsigned char*)param,
			     &(hpe_dev[devnr].estate.lim), 1);
		break;

	case HPE_GETSTATE:
		copy_to_user((struct hpe_engine_state*)param,
			     &(hpe_dev[devnr].estate),
			     sizeof (struct hpe_engine_state));
		break;
		


	case HPE_RENEWSTATE:
		hpe_set_speed(devnr, hpe_dev[devnr].estate.speed);
		copy_to_user((struct hpe_engine_state*)param,
			     &(hpe_dev[devnr].estate),
			     sizeof (struct hpe_engine_state));
		break;

	case HPE_GETCLOCK:
		data = hpe_get_clock();
		copy_to_user((double*)param, &data, sizeof (double));
		break;

	case HPE_SETRAMP:
		/*printk (HPE_NAME": HPE_SETRAMP, and NOT stopping engines!\n");*/
		hpe_set_speed (devnr, 0.0);
		hpe_dev[devnr].ramp_pos = 0;
		copy_from_user(&(hpe_dev[devnr].accel_ramp),
			       (struct hpe_accel_ramp*)param,
			       sizeof (struct hpe_accel_ramp));
		printk ("ramp size: %d items\n",
			hpe_dev[devnr].accel_ramp.size);
		printk ("ramp time slice: %d msec\n",
			(int)(hpe_dev[devnr].accel_ramp.tslice*1000.0));
		printk ("start speed: %d rad/sec\n",
			(int)(hpe_dev[devnr].accel_ramp.ramp[1].speed*100.0));
		printk ("maximum speed: %d rad/sec\n",
			(int)(hpe_dev[devnr].accel_ramp.ramp[hpe_dev[devnr].accel_ramp.size-1].speed*100.0));
		break;

	case HPE_GETRAMP:
		copy_to_user((struct hpe_accel_ramp*)param,
			     &(hpe_dev[devnr].accel_ramp),
			     sizeof (struct hpe_accel_ramp));
		break;

	case HPE_SETPOSFAC:
		if (minor%2) return -EIO;
		copy_from_user (&(hpe_dev[devnr].pos_fac), (double *)param, sizeof(double));
		break;

	case HPE_GETPOSFAC:
		copy_to_user((double*)param, &(hpe_dev[devnr].pos_fac), sizeof (double));
		break;

	case HPE_SETSPDFAC:
		if (minor%2) return -EIO;
		copy_from_user (&(hpe_dev[devnr].spd_fac), (double *)param, sizeof(double));
		break;

	case HPE_GETSPDFAC:
		copy_to_user((double*)param, &(hpe_dev[devnr].spd_fac), sizeof (double));
		break;

	case HPE_RESET:
		if (minor%2) return -EIO;
		hpe_reset_pic ();
		break;

	default:
		printk (HPE_NAME": ioctl %d not implemented\n", cmd);
		break;
	}

	return ret;
}

/*
 * module loading stuff
 */
 
static struct file_operations hpe_fops = {
#ifdef MODULE
	owner:          THIS_MODULE,
#endif
	read:           (void *) hpe_read,   /* read */
	write:          (void *) hpe_write,  /* write */
	ioctl:          (void *) hpe_ioctl,  /* ioctl */
	open:           (void *) hpe_open,   /* open */
	release:        (void *) hpe_release /* release */
};

#ifdef MODULE
/*
 * init the module (if loaded as module)
 */
int init_module (void)
{
	return hpe_init ();
}

/*
 * removes the module
 */
void cleanup_module (void)
{
	devfs_unregister (devfs_handle);
	if (devfs_unregister_chrdev (HPE_MAJOR, HPE_NAME) < 0) {
		printk (HPE_NAME": unregister failed\n");
	}
	
	printk (HPE_NAME": have a nice day\n");
}
#endif /* MODULE */

int hpe_init (void)
{
	
	int i, max, j;
	double d, t;
	char name[100];
	printk ("\n");
	printk (HPE_NAME": version %s\n", HPE_VERSION);

	printk (HPE_NAME": initialising... ");

	hpe_reset_pic ();

#define WAITCNT 100000
	
	/* 
	 * ask for current time (check existance of the controller)
	 */
	for (i = WAITCNT; i; i--); /* some delay, to have an interesting time value ;) */
	for (i = WAITCNT; i; i--);	

	d = 0;
	for (j=0; j<20; j++) {
		if (!j) { SetPA0(0); }
		else    { SetPA0(1); }
		SetPA1(0);
		SetPSTR(0);
		i=WAITCNT; while (--i && (GetPIC & 0x80)) { if (!i%10000) printk ("X "); };
		if (!i)	{ printk (" failed, hardware not found\n"); return -EIO; };
		SetPSTR(1);

		d *= 16.0;
		i=WAITCNT; while (--i && !(GetPIC & 0x80) ) { if (!i%1000) printk ("X "); };
		if (!i) { printk (" failed, hardware not found\n"); return -EIO; };
		d += (double)(GetPIC&0x0f);
	}
	t = d/39062.5;

	printk ("ok (controller time: %d.%d)\n",
		(int)t,
		(int)((t-(double)((int)t)) *1000.0));


	/* registering device to devfs */
	HPEMajor = HPE_MAJOR;
	if (0 != devfs_register_chrdev (HPE_MAJOR, HPE_NAME, &hpe_fops)) {
		printk (HPE_NAME": device registration (major %d) failed\n",
			HPEMajor);
		return -EIO;
	}
	printk (HPE_NAME": registered with major %d\n",
		HPEMajor);

	devfs_handle = devfs_mk_dir (NULL, "hpe", NULL);
	for (i = 0; i < 4; i++) {
		sprintf (name, "ctrl%d", i);
		devfs_register (devfs_handle, name, DEVFS_FL_DEFAULT, HPE_MAJOR, i*2,
				S_IFCHR | S_IRUGO | S_IWUGO, &hpe_fops, NULL);
		sprintf (name, "stat%d", i);
		devfs_register (devfs_handle, name, DEVFS_FL_DEFAULT, HPE_MAJOR, i*2+1,
				S_IFCHR | S_IRUGO, &hpe_fops, NULL);
	}

	
	max=sizeof(struct hpe_devinfo)*4;
	
	/* bzero, set defaults */
	for (i=0; i < max; i++) {
		((unsigned char*)(*(&hpe_dev)))[i] = 0;
		hpe_dev[i].spd_fac = HPEDefSpdFactor;
		hpe_dev[i].pos_fac = HPEDefPosFactor;
	}

	/* make a one-size-fits-all ramp:
	 * 30 steps
	 * 3.0 sec total acceleration time (zero to maximum speed)
	 * ~9.5 full turns per second as maximum speed
	 * don't rely on this one, it's just a toy.
	 */
	hpe_mkramp (&(hpe_dev[0].accel_ramp), 30, 3.0, 60.0);
	hpe_mkramp (&(hpe_dev[1].accel_ramp), 30, 3.0, 60.0);
	hpe_mkramp (&(hpe_dev[2].accel_ramp), 30, 3.0, 60.0);
	hpe_mkramp (&(hpe_dev[3].accel_ramp), 30, 3.0, 60.0);
	
	return 0;
}

/*
 * hardware stuff
 */

/*
 * reset the hardware
 */
void hpe_reset_pic (void)
{
	int i;

	HPEconIO=0x00;  /* contents of the i/o register */
	HPEconAux=0x00; /* contents of the aux register */
	HPEconDac=0x00; /* contents of the dac register */
	
	SetReset(0);  /* reset the engine micro-controller */
	SetRESPIC(0); /* reset the timer micro-controller */

	for (i = 0; i < 4; i++) {
		SetRW(i,0);
		SetRW(i,1);
	}
	
	for (i = 0; i < 4; i++) {
		SetDI(i,0);
		SetDI(i,1);
		
		hpe_dev[i].estate.speed  = 0.0;
		hpe_dev[i].estate.pos    = 0.0;
		hpe_dev[i].estate.time   = 0.0;
		hpe_dev[i].estate.lim    = 0;
	}

	/* init timer interface lines */
	SetPSTR(0); SetPSTR(1);
	SetPA0(0);  SetPA0(1);
	SetPA1(0);  SetPA1(1);

	/* set the lines free -- we're on line ;-) */
	SetRESPIC(1);
	SetReset(1);

	/* initialize DAC */
	SetDAC(0);
}

/*
 * write a byte to the dac (you can use this for 8-bit speech output, for example)
 */
inline void hpe_put_dac (unsigned char c)
{
	SetDAC(c);
}

/*
 * get the device internal time
 *
 * this time _is_ _not_ the real time!
 * it's merely a counter which increases 
 * every 25.6us (precisely)
 *
 * the delay of the function is 512us (right?)
 *
 * the function returns the time in seconds
 *
 */
double hpe_get_clock (void)
{
	int i;
	double d;

	d = 0;
	for (i=0; i<20; i++) {
		if (!i) { SetPA0(0); }
		else    { SetPA0(1); }
		SetPA1(0);
		SetPSTR(0);
		while ((GetPIC & 0x80)) {};
		SetPSTR(1);

		d *= 16.0;
		while ( !(GetPIC & 0x80) ) {};
		d += (double)(GetPIC&0x0f);
	}

	return d/39062.5;
}

/*
 * fix the position (=> add the suplimentary bits from LEFT to RITE
 */
/*  inline void hpe_normpos (struct hpe_engine_pos *pos) */
/*  { */
/*  	pos->left += pos->rite>>24; */
/*  	pos->rite = pos->rite<<8>>8; */
/*  } */

inline void hpe_send_bit(unsigned char nr, unsigned char bit)
{
	while(!GetClk(nr)) { };
	SetDI(nr,bit);
	while(GetClk(nr))  { };
}

inline unsigned char hpe_recv_bit (unsigned char nr)
{
	unsigned char c;
	while(!GetClk(nr)) {};
	c=GetOut(nr);
	while(GetClk(nr)) {};
	return c;
}

/*
 * wait routine with an accuracy of about 25.6us and
 */
inline void hpe_wait (double sec)
{
	double t = hpe_get_clock ();
	while (hpe_get_clock()-t < sec) {};
}

/*
 * talk to the engine
 *
 * nr:     the engine nr
 * speed:  the speed
 * powd:   power down (0=>50%, 1=>100%)
 * pos:    step of the engine
 * lim:    (1=>upper switch, 2=>lower switch)
 *
 * WARNING !! One send/recv loop _must_ be done in less than 12us !!
 */

void hpe_ctrl_engine (uint8_t nr, double fspeed,
		     struct hpe_engine_state *stat)
{
	uint32_t pos;

	cli();
	hpe_engine_send (nr, fspeed, fspeed>1.0 ? 1 : 1); /* don't set power down */
	hpe_engine_recv (nr, &pos, &(stat->lim));
	stat->speed = fspeed;
	stat->time  = hpe_get_clock();

        /* 
	 * mapping position from 0..x to -x/2..x/2
	 * works for 24.8 bit controller
	 */       
	stat->pos = HPE_POS_RAD ( hpe_dev[nr], POSITION_MAP(pos) );
	
	sti();
}

/*
 * sets the power-down bit (0: 50%  1: 100%)
 * and engine speed (degree/second)
 */
inline void hpe_engine_send (uint8_t engine_nr, double fspeed, int power_down)
{
	int i;
	uint32_t speed = HPE_SPD_DEV(hpe_dev[engine_nr],fspeed);

	/* set write */
	while (!GetClk(engine_nr)) { };
	SetRW(engine_nr,0);
	while (GetClk(engine_nr)) { };

	hpe_send_bit (engine_nr, 0);                              /* dummy */
	hpe_send_bit (engine_nr, power_down!=0);                  /* power down bit */
	for (i=0; i < 32; i++)
		hpe_send_bit (engine_nr, (speed>>(31-i)) & 0x01); /* speed */
	hpe_send_bit (engine_nr, 0);	                          /* dummy */
}

/*
 * receives the engine position (steps) and the limits
 * hi_lim and lo_lim are arbitrary names, they don't
 * say anything about the position of the hit sensors.
 * they might be next to each other, in which case both
 * hi_lim and lo_lim would be set, which is
 * not necessarily an error!
 *
 * Beware: right after a send must follow one receive!
 *         (hardware constraints)
 */
inline void hpe_engine_recv (uint8_t engine_nr, uint32_t *steps, uint8_t *lim)
{
	int i;
	uint32_t pos=0;

	/* set read */
	while (!GetClk(engine_nr)) {};
	SetRW(engine_nr,1);
	while (GetClk(engine_nr)) {};

	hpe_recv_bit (engine_nr); 	                     /* dummy */
	for (i=0; i < 32; i++)
		pos = (pos<<1) | hpe_recv_bit (engine_nr);   /* position */
	for (i=0; i < 8; i++)
		*lim = (*lim<<1) | hpe_recv_bit (engine_nr); /* limits */

	*steps = pos;
}

/*
 * this function traverses the speed ramp in ascendent direction (as opposed to hpe_dec_speed)
 *
 * there are several cases:
 * 1. we have a positive small speed and we want a positive bigger one
 * 2. we have a negative big speed and we want a negative smaller one
 *    or a positive one
 *
 * strategy:
 *
 * we want to reach the speed 'speed'. first we check if 'speed' is in
 * the valid boundaries. if it's not, we set truncate it to the maximum
 * (or minimum) allowed. then we run up the speed ramp (or negative down)
 * until we reach the curren_ramp_speed < speed < next_ramp_speed. then
 * we finish the loop and finally set the requested 'speed'.
 *
 * returns: the distance it took to reach the given speed
 */
inline double hpe_inc_speed (uint8_t engnr, double speed)
{
	double spd;
	double dist = 0.0;
	double t1 = 0.0, t2 = 0.0;
	int ri = hpe_dev[engnr].ramp_pos;
	struct hpe_accel_ramp *r = &(hpe_dev[engnr].accel_ramp);

	if (speed > r->ramp[r->size-1].speed) speed = r->ramp[r->size-1].speed;
	if (speed < -r->ramp[r->size-1].speed) speed = -r->ramp[r->size-1].speed;

	if (hpe_dev[engnr].last_move == HPE_MOV_DECR) ri--;

	while (1) {
		ri++;
		
		if (ri < 0) spd = -(r->ramp[-ri].speed);
		else spd = r->ramp[ri].speed;

		/*printk ("ri: %d\nspd: %d\nspeed: %d\n", ri, (int)(spd*100.0), (int)(speed*100.0));*/
		
		if (spd >= speed || ri == r->size) {
			ri--; /* undo the last try */
			goto last_step;
		}
	
		/*printk ("increase to: %d\n", (int)(spd*100));*/
		hpe_set_speed (engnr, spd);
		t2 = hpe_dev[engnr].estate.time;
		if (t1!=0.0) dist += (t2-t1) * spd;
		else dist += r->tslice*spd;
		t1 = t2;
		//dist += spd*r->tslice;
		hpe_wait (hpe_dev[engnr].accel_ramp.tslice);
	}
 last_step:
	/*printk ("final increase to: %d\n", (int)(speed*100));*/
	hpe_dev[engnr].ramp_pos = ri;
	hpe_dev[engnr].last_move = HPE_MOV_INCR;
	/*printk ("new ramp_pos: %d\n", ri);*/
	hpe_set_speed (engnr, speed);
	
	return dist;
}

inline double hpe_dec_speed (uint8_t engnr, double speed)
{
	double spd;
	double dist = 0.0;
	double t1 = 0.0, t2 = 0.0;
	int ri = hpe_dev[engnr].ramp_pos;          /* get the old position in the ramp */
	struct hpe_accel_ramp *r = &(hpe_dev[engnr].accel_ramp);
	
	if (speed > r->ramp[r->size-1].speed) speed = r->ramp[r->size-1].speed;
	if (speed < -r->ramp[r->size-1].speed) speed = -r->ramp[r->size-1].speed;

	if (hpe_dev[engnr].last_move == HPE_MOV_INCR) ri++;

	while (1) {
		ri--;

		if (ri < 0) spd = -r->ramp[-ri].speed;
		else spd = r->ramp[ri].speed;
		
		if (spd <= speed || ri == -r->size) {
			ri++; /* undo the last try */
			goto last_step;
		}
		
		/*printk ("decrease to: %d\n", (int)(spd*100));*/
		hpe_set_speed (engnr, spd);
		t2 = hpe_dev[engnr].estate.time;
		if (t1 != 0.0) dist += (t2-t1) * spd;
		else dist += r->tslice*spd;		
		t1 = t2;
		//dist += spd*r->tslice;
		hpe_wait (hpe_dev[engnr].accel_ramp.tslice);
	}
 last_step:
	/*printk ("final decrease to: %d\n", (int)(speed*100));*/
	hpe_dev[engnr].ramp_pos = ri;
	hpe_dev[engnr].last_move = HPE_MOV_DECR;
	/*printk ("new ramp_pos: %d\n", ri);*/
	hpe_set_speed (engnr, speed);
	
	return dist;
}

/*
 * create an acceleration ramp with the given 'size' (steps),
 * acceleration time from zero to max-speed (max_t) and a given maximum speed (max_sp).
 */
double sfunc (double x) { return x; } /* function used to calculate ramp -- normally exp(x) */
void hpe_mkramp (struct hpe_accel_ramp *ramp, int size, double max_t, double max_sp)
{
	int i;
	double t = 0.0;
	double speed;
	ramp->size = size;
	ramp->tslice = max_t/(double)size;

	for (i = 0; i < size; i++, t+= ramp->tslice) {
		speed = max_sp*(1.0-1.0/sfunc(t));
		ramp->ramp[i].speed = speed;
		ramp->ramp[i].dist = ramp->tslice * speed;
	}
}

/*
 * run up the ramp to the given speed;
 */
double hpe_goto_speed (uint8_t engnr, double gps)
{
	printk ("setting to gps: %d\n", (int)(gps*100.0));
	if (gps > hpe_dev[engnr].estate.speed)
		return hpe_inc_speed (engnr, gps);
	else if (gps < hpe_dev[engnr].estate.speed)
		return hpe_dec_speed (engnr, gps);
	else return 0.0;
}


/*
 * we have to 'slowly' increase the speed by HPE_MAX_ACCEL to our current speed
 * we'll do it smooth, say 100 times/sec
 */
/*  void hpe_goto_speed (uint8_t engnr, double gps) */
/*  { */
/*  	double dt = HPEMaxVDelta/HPEAccel; *//* time slice */
/*  	double dv = HPEMaxVDelta;          *//* speed slice */

/*  	double speed = hpe_dev[engnr].estate.speed; */

/*  	printk ("slices*100: %d, %d\n", (int)(dt*100.0), (int)(dv*100.0)); */

/*  	if (gps > speed) { */

/*  		for (; speed < gps-dv; speed += dv) { */
/*  			hpe_set_speed (engnr, speed); */
/*  			hpe_wait (dt); */
/*  		} */

/*  	} else { */

/*  		for (; speed > gps+dv; speed-=dv) { */
/*  			hpe_set_speed (engnr, speed); */
/*  			hpe_wait (dt); */
/*  		} */
/*  	} */

	/*
	 * be sure we reached the final speed
	 * (sub-delta correction)
	 */
/*  	hpe_set_speed (engnr, gps); */
/*  } */

/*
 * almost like hpe_goto_speed, with the difference that goto_speed
 * will smoothly increase speed, and set_speed will simply set it
 * and hope the engine can handle it.
 */
inline void hpe_set_speed (uint8_t engine, double gps)
{
	hpe_ctrl_engine (engine, gps, &(hpe_dev[engine].estate));
}

#undef __BUILDING_HPE__



