/*
-----------------------------------------------------------

   amath.c / amath.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/amath.h>
#include <math.h>

double sn(double x)
{
	return (sin(x * RAD));
}

double cs(double x)
{
	return (cos(x * RAD));
}

double tn(double x)
{
	return (tan(x * RAD));
}

double asn(double x)
{
	if (fabs(x) == 1.0)
		return (90.0 * x);
	else {
		if (fabs(x) > EPS)
			return (atan(x / sqrt((1.0 - x) * (1.0 + x))) / RAD);
		else
			return (x / RAD);
	}
}

double acs(double x)
{
	double c = 90.0;

	if (fabs(x) == 1.0)
		return (c - x * c);
	else {
		if (fabs(x) > EPS)
			return (c - atan(x / sqrt((1.0 - x) * (1.0 + x))) / RAD);
		else
			return (c - x / RAD);
	}
}

double atn(double x)
{
	return (atan(x) / RAD);
}

double cubr(double x)
{
	if (x == 0.0)
		return (0.0);
	else
		return (exp(log(x) / 3.0));
}

double atn2(double y, double x)
{
	double ax, ay, phi;

	if ((x == 0.0) && (y == 0.0))
		return (0.0);
	else {
		ax = fabs(x);
		ay = fabs(y);
		if (ax > ay)
			phi = atan(ay / ax) / RAD;
		else
			phi = 90.0 - atan(ax / ay) / RAD;
		if (x < 0.0)
			phi = 180.0 - phi;
		if (y < 0.0)
			phi = -phi;
		return (phi);
	}
}

double frac(double x)
{
	x = x - (long) x;
	if (x < 0.0)
		x = x + 1.0;
	return (x);
}

void quad(double yminus, double y0, double yplus, double *xe, double *ye,
	  double *zero1, double *zero2, int *nz)
{
	double a, b, c, dis, dx;

	nz = 0;
	a = 0.5 * (yminus + yplus) - y0;
	b = 0.5 * (yplus - yminus);
	c = y0;
	*xe = -b / (2.0 * a);
	*ye = (a * *xe + b) * *xe + c;
	dis = b * b - 4.0 * a * c;
	if (dis >= 0) {
		dx = 0.5 * sqrt(dis) / fabs(a);
		*zero1 = *xe - dx;
		*zero2 = *xe + dx;
		if (fabs(*zero1) <= 1.0)
			*nz = *nz++;
		if (fabs(*zero2) <= 1.0)
			*nz = *nz++;
		if (*zero1 <= 1.0)
			*zero1 = *zero2;
	}
}
