/*
-----------------------------------------------------------

   atime.c / atime.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/atime.h>
#include <astro/amath.h>

double mjd(int day, int month, int year, double hour)
{
	double a;
	int b;

	a = 10000.0 * year + 100.0 * month + day;
	if (month <= 2) {
		month = month + 12;
		year = year - 1;
	};
	if (a <= 15821004.1)
		b = -2 + (int) ((year + 4716) / 4) - 1179;
	else
		b = (int) (year / 400) - (int) (year / 100) + (int) (year / 4);
	a = 365.0 * year - 679004.0;
	return (a + b + (int) (30.6001 * (month + 1)) + day + hour / 24.0);
}

double jd(int day, int month, int year, double hour)
{
	return (mjd(day, month, year, hour) + 2400000.5);
}

void caldat(double mjd, int *day, int *month, int *year, double *hour)
{
	int b, d, f;
	double jd, jd0, c, e;

	jd = mjd + 2400000.5;
	jd0 = (long) (jd + 0.5);	/* Trunc?? */
	if (jd0 < 2299161.0)
		c = jd0 + 1524.0;
	else {
		b = (int) ((jd0 - 1867216.25) / 36524.25);
		c = jd0 + (b - (b / 4)) + 1525.0;
	};
	d = (int) ((c - 122.1) / 365.25);
	e = 365.0 * d + (d / 4);
	f = (int) ((c - e) / 30.60019);
	*day = (int) (c - e + 0.5) - (int) (30.6001 * f);
	*month = f - 1 - (12 * (int) (f / 14));
	*year = d - 4715 - (int) ((7 + *month) / 10);
	*hour = 24.0 * (jd + 0.5 - jd0);
}

double gmst(double mjd)
{
	double mjd0, t, ut, g;

	mjd0 = (double) (long) mjd;
	ut = (mjd - mjd0) * 24.0;
	t = (mjd0 - 51544.5) / 36525.0;
	g = (6.697374558 + 1.0027379093 * ut +
	    (8640184.812866 + (0.093104 - 6.2E-6 * t) * t) * t / 3600.0);
	return (24.0 * frac((g) / 24.0));
}

double lmst(double mjd, double lambda)
{
	return (24.0 * frac((gmst(mjd) - lambda / 15.0) / 24.0));
}
