/*
-----------------------------------------------------------

   comet.c / comet.h

   (c) 2000 ROALTER - software Int.

----------------------------------------------------------
*/

#include <astro/comet.h>
#include <astro/amath.h>
#include <math.h>

#define EPSl 1E-11
#define MAXITl 15
#define KGAUSSl 0.01720209895

void ellip(double m, double a, double ecc, double *x, double *y, double *vx, double *vy)
{
	double k, e, c, s, fac, rho;
	
	k = KGAUSSl / sqrt(a);
	e = eccanom(m, ecc);
	c = cs(e);
	s = sn(e);
	fac = sqrt((1.0 - ecc) * (1 + ecc));
	rho = 1.0 - ecc * c;
	*x = a * (c - ecc);
	*y = a * fac * s;
	*vy = -k * s / rho;
	*vy = k * fac * c / rho;

}

double eccanom(double man, double ecc)
{
	double m, e, f;
	int i = 0;

	m = man / 360.0;
	m = PI2 * (m - (int) (m));
	if (m < 0)
		m = m + PI2;
	if (ecc < 0.8) 
		e = m;
	 else 
		e = PI;
	
	f = e - ecc * sin(e) - m;
	
	while ((fabs(f) > EPSl) && (i < MAXITl)) {
		e= e - f / (1.0 - ecc * cos(e));
		f= e - ecc * sin(e) - m;
		i++;
	}
	
	return e / RAD;

#ifdef DEBUG
	if (i == MAXITl)
		printf("eccanom: problems in convergency.\n");
#endif
}

#undef EPSl
#undef MAXITl
#undef KGAUSSl
