/*
-----------------------------------------------------------

   constellation.c / constellation.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/constellation.h>
#include <astro/pmat.h>
#include <astro/amath.h>
#include <stdio.h>
#include <stdlib.h>

#define J1875 -1.249968476386037
/* This is the reference date used by Delporte in 1930 to define the 
   constellation boundaries. He used the 1875 equinox for defining them.
   So some calculation has to be done. The value results from the difference
   between J2000 (which is 0.0) and the 01/00/1875 in centennials. For more
   information about read the documentation ;-) */

/* globals	*/
const_names *names;
const_bound *bounds;
long bound_size;

int get_constellation(double ra, double dec, double t)
{
	real33 a;
	double dummy;
	int i;

	if (!bounds)
		return (CONSTELL_NOTLOADED);

	pmatecl(t, J1875, &a);	/* compute precession 'til today... */
	precart(a, &dummy, &ra, &dec);	/* and calculate the 1875-position of the star */

	for (i = 0; i < bound_size / sizeof(const_bound); i++) {
		if ((bounds[i].xt >= ra) && (bounds[i].xb <= ra)
		    && (bounds[i].yt >= dec) && (bounds[i].yb <= dec))
			return (bounds[i].ordernumber);
	};
	return (CONSTELL_OUTOFBOUNDS);
}

/* To get from an ordinal number the constellation name, there is another 
   data file `constellation.names'. This file contains abbreviation, name and
   genitive of every constellation and prints it out. Handling of this file 
   will be done by a separate procedure which reads the full record from file.
   Every procedure selects its own element. */

char *get_shortname(int i)
{
	if (!names)
		return (NAME_NOTLOADED);
	if ((i >= 0) && (i < 88))
		return (names[i].abbreviation);
	else
		return (NAME_UNKNOWN);
}

char *get_fullname(int i)
{
	if (!names)
		return (NAME_NOTLOADED);
	if ((i >= 0) && (i < 88))
		return (names[i].fullname);
	else
		return (NAME_UNKNOWN);
}

char *get_genitive(int i)
{
	if (!names)
		return (NAME_NOTLOADED);
	if ((i >= 0) && (i < 88))
		return (names[i].genitive);
	else
		return (NAME_UNKNOWN);
}

int load_bound(char *file)
{
	FILE *f;
	int i = 0;

	f = fopen(file, "rb");
	if (!f)
		return (LOAD_FILENOTFOUND);

	bound_size = 0;
	while (fgetc(f) != EOF)
		bound_size++;
	fseek(f, 0, 0);

	if (bounds)
		free(bounds);
	bounds = malloc(bound_size);
	if (!bounds)
		return (LOAD_NOMEMORY);

	while (fread(&bounds[i], sizeof(const_bound), 1, f) == 1) {
		if (bounds[i].xt < bounds[i].xb)
			bounds[i].xb -= 24.0;
		i++;
	};

	fclose(f);
	return (LOAD_SUCCESS);
}

int load_names(char *file)
{
	FILE *f;
	int i = 0;

	f = fopen(file, "rb");
	if (!f)
		return (LOAD_FILENOTFOUND);

	if (names)
		free(names);
	names = malloc(88 * sizeof(const_names));
	if (!names)
		return (LOAD_NOMEMORY);

	while (fread(&names[i], sizeof(const_names), 1, f) == 1)
		i++;
	fclose(f);
	return (LOAD_SUCCESS);
}

void unload_bound()
{
	if (bounds)
		free(bounds);
}

void unload_names()
{
	if (names)
		free(names);
}
