/*
-----------------------------------------------------------

   coord.c / coord.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/coord.h>
#include <astro/amath.h>
#include <math.h>

void cart(double r, double theta, double phi, double *x, double *y, double *z)
{
	double rcst;

	rcst = r * cs(theta);
	*x = rcst * cs(phi);
	*y = rcst * sn(phi);
	*z = r * sn(theta);
}

void polar(double x, double y, double z, double *r, double *theta, double *phi)
{
	double rho;

	rho = x * x + y * y;
	*r = sqrt(rho + z * z);
	*phi = atn2(y, x);
	if (*phi < 0)
		*phi = *phi + 360.0;
	rho = sqrt(rho);
	*theta = atn2(z, rho);
}

void ddd(int d, int m, double s, double *dd)
{
	double sign;

	if ((d < 0) || (m < 0) || (s < 0))
		sign = -1.0;
	else
		sign = 1.0;
	*dd = sign * (abs(d) + abs(m) / 60.0 + fabs(s) / 3600.0);
}

void dms(double dd, int *d, int *m, double *s)
{
	double d1;

	d1 = fabs(dd);
	*d = (int) d1;
	d1 = (d1 - *d) * 60.0;
	*m = (int) d1;
	*s = (d1 - *m) * 60.0;
	if (dd < 0.0) {
		if (*d != 0)
			*d = -*d;
		else {
			if (*m != 0)
				*m = -*m;
			else
				*s = -*s;
		}
	}
}
