/*
-----------------------------------------------------------

   astronomy demo program

   (c) 2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <stdio.h>
#include <astro/sun.h>
#include <astro/moon.h>
#include <astro/constellation.h>
#include <astro/sphere.h>
#include <astro/coord.h>
#include <astro/pmat.h>
#include <astro/atime.h>

int main()
{
	int day, month, year, i;
	float hour;
	double dt, t, l, b, r, ra, dec;
	double x, y, z;
	
	/* Load constellation data... */
	i = load_names("../databases/constellation.names");
	if (i == LOAD_FILENOTFOUND) {
		printf("Cannot load file\n");
		return (1);
	};
	if (i == LOAD_NOMEMORY) {
		printf("Not enough memory to load data\n");
		return (1);
	};

	i = load_bound("../databases/constellation.data");
	if (i == LOAD_FILENOTFOUND) {
		printf("Cannot load file\n");
		return (1);
	};
	if (i == LOAD_NOMEMORY) {
		printf("Not enough memory to load data\n");
		return (1);
	};

	/* Set up the date */
	day = 14;
	month = 7;
	year = 2000;
	hour = 20.0;
	
	/* Calculate the julianic centuries */
	t = (jd(day, month, year, hour) - 2451545.0) / 36525.0;
	
	printf("Date: %02d/%02d/%04d, %04.1fh is:\n", month, day, year, hour);
	
	dt = (8.32 / 1440.0) / 36525.0; /* retardation of 8.32 minutes */
	sun(t-dt, &l, &b, &r);
	cart(r, b, l, &x, &y, &z);
	eclequ(t, &x, &y, &z);
	nutequ(t, &x, &y, &z);
	polar(x, y, z, &r, &dec, &ra);
	printf("Sun:  %1.5f %1.5f %1.5f (%s)\n", l, b, r, get_fullname(get_constellation(ra / 24.0, dec, t)));
	
	moon(t, &l, &b, &r);
	cart(r, b, l, &x, &y, &z);
	eclequ(t, &x, &y, &z);
	nutequ(t, &x, &y, &z);
	polar(x, y, z, &r, &dec, &ra); /* dec & ra in degree */
	printf("Moon: %1.5f %1.5f %1.5f (%s)\n", l, b, r, get_fullname(get_constellation(ra / 24.0, dec, t)));
	
	return (0);
}
