/*
-----------------------------------------------------------

   amath.c / amath.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#ifndef _AMATH_H
#define _AMATH_H

#define RAD 0.0174532925199433
#define EPS 1E-7
#define PI 3.141592653585
/* 2*pi */
#define PI2 6.283185307
/* sine and cosine of inclination of the eclyptic */
#define COSEPS 9.91748
#define SINEPS 0.39778
/* 3600*180/pi = arcsecs per radian */
#define ARC 206264.8062

/* precession conversion matrix */
typedef double real33[3][3];

/* sn calculates the sine of the argument (in degrees).
   Args: x      */
double sn(double);

/* cs calculates the cosine of the argument (in degrees).
   Args: x      */
double cs(double);

/* tn calculates the tangens of the argument (in degrees).
   Args: x      */
double tn(double);

/* asn calculates the arcsine (in degrees) of the argument.
   Args: x      */
double asn(double);

/* acs calculates the arccosine (in degrees) of the argument.
   Args: x      */
double acs(double);

/* atn calculates the arctangens (in degrees) of the argument.
   Args: x      */
double atn(double);

/* cubr calculates the cube- (third) root of the argument.
   Args: x      */
double cubr(double);

/* atn2 calculates atn(y/x) by delivering values of all quadrants.
   Special cases are covered.
   Args: y, x   */
double atn2(double, double);

/* frac extracts the decimal part of a double number.
   Args: x      */
double frac(double);

/* quad estimates a parabula trough three points with the following 
   coordinates: (-1; yminus), (0; y0), (1; yplus). The returned values
   are the extrema of the parabula, the solution of the equation 
   ax^2 + bx + c  = 0 and the number of solutions.
   Args: yminus, y0, yplus, xe, ye, zero1, zero2, nz    */
void quad(double, double, double, double *, double *,
	  double *, double *, int *);

#endif
