/*
-----------------------------------------------------------

   constellation.c / constellation.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#ifndef _CONSTELLATION_H
#define _CONSTELLATION_H

/* Error return values for the load_ and unload_ functions */
#define LOAD_SUCCESS           0
#define LOAD_FILENOTFOUND    (-1)
#define LOAD_NOMEMORY        (-2)

/* Error return values for the get_constellation function */
#define CONSTELL_NOTLOADED   (-1)
#define CONSTELL_OUTOFBOUNDS (-2)

/* Error return values for the get_*name* functions */
#define NAME_NOTLOADED    "empty"
#define NAME_UNKNOWN    "unknown"

/* Form of the structure used to save the constellation data in the
   file `constellation.data'. */
typedef struct {
	double xt, yt;
	double xb, yb;
	short int ordernumber;
} const_bound;

/* Form of the structure used to save constellation names and other
   informations in the file `constellation.names'. */
typedef struct {
	char abbreviation[4];
	char fullname[21];
	char genitive[22];
} const_names;

/* get_constellation delivers the ordernumber of the constellation
   in which lies the indicated star with the coordinates ra:dec at a 
   given time t in julianic centennials.
   Args: ra, dec, t     */
int get_constellation(double, double, double);

/* get_shortname delivers a three char long string with the international
   abbreviation of a constellation in the correct written form: UMi, Ori;
   constellations are ordered alphabetically. range of i is between 1 and
   88. 
   get_fullname delivers the full name of the same constellation:
   Ursa Minoris, Orion, Ophiuchus, ... 
   get_genitive delivers the genitive full name of the constellation:
   Ursae Minoris, Orionis, Geminii, Tauri, Aquarii
   Args: i              */
char *get_shortname(int);
char *get_fullname(int);
char *get_genitive(int);

/* load_bound and unload_bound are two function to load the boundary file 
   specified in the parameter list and discard it (usually 
   `constellation.data').
   load_names and unload_names load the constellation names file into 
   memory and discard the data (file usually `constellation.names').
   Args: file           */
int load_bound(char *);
void unload_bound();

int load_names(char *);
void unload_names();

#endif
