/*
-----------------------------------------------------------

   pmat.c / pmat.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/pmat.h>
#include <math.h>

void pmatecl(double t1, double t2, real33 * a)
{
	double dt, ppi, pi, pa, c0, s0, c1, s1, c2, s2;

	dt = t2 - t1;
	ppi = 174.876383889 + (((3289.4789 + 0.60622 * t1) * t1) +
	      ((-869.8089 - 0.50491 * t1) + 0.03536 * dt) * dt) / 3600.0;
	pi = ((47.0029 - (0.06603 - 0.000598 * t1) * t1) +
	((-0.03302 + 0.000598 * t1) + 0.000060 * dt) * dt) * dt / 3600.0;
	pa = ((5029.0966 + (2.22226 - 0.000042 * t1) * t1) +
	 ((1.11113 - 0.000042 * t1) - 0.000006 * dt) * dt) * dt / 3600.0;
	c0 = cs(ppi + pa);
	c1 = cs(pi);
	c2 = cs(ppi);
	s0 = sn(ppi + pa);
	s1 = sn(pi);
	s2 = sn(ppi);
	(*a)[0][0] = c0 * c2 + s0 * c1 * s2;
	(*a)[0][1] = c0 * s2 - s0 * c1 * c2;
	(*a)[0][2] = -s0 * s1;
	(*a)[1][0] = s0 * c2 - c0 * c1 * s2;
	(*a)[1][1] = s0 * s2 + c0 * c1 * c2;
	(*a)[1][2] = c0 * s1;
	(*a)[2][0] = s1 * s2;
	(*a)[2][1] = -s1 * c2;
	(*a)[2][2] = c1;
}

void pmatequ(double t1, double t2, real33 * a)
{
	double dt, zeta, z, theta, c0, s0, c1, s1, c2, s2;

	dt = t2 - t1;
	zeta = ((2306.2181 + (1.39656 - 0.000139 * t1) * t1) +
	 ((0.30188 - 0.000345 * t1) + 0.017998 * dt) * dt) * dt / 3600.0;
	z = zeta + ((0.79280 + 0.000411 * t1) + 0.000205 * dt) * dt * dt / 3600.0;
	theta = ((2004.3109 - (0.85330 + 0.000217 * t1) * t1) -
	 ((0.42665 + 0.000217 * t1) + 0.041833 * dt) * dt) * dt / 3600.0;
	c0 = cs(z);
	c1 = cs(theta);
	c2 = cs(zeta);
	s0 = sn(z);
	s1 = sn(theta);
	s2 = sn(zeta);
	(*a)[0][0] = -s0 * s2 + c0 * c1 * c2;
	(*a)[0][1] = -s0 * c2 - c0 * c1 * s2;
	(*a)[0][2] = -c0 * s1;
	(*a)[1][0] = c0 * s2 + s0 * c1 * c2;
	(*a)[1][1] = c0 * c2 - s0 * c1 * s2;
	(*a)[1][2] = -s0 * s1;
	(*a)[2][0] = s1 * c2;
	(*a)[2][1] = -s1 * s2;
	(*a)[2][2] = c1;
}

void precart(real33 a, double *x, double *y, double *z)
{
	double u, v, w;

	u = a[0][0] * *x + a[0][1] * *y + a[0][2] * *z;
	v = a[1][0] * *x + a[1][1] * *y + a[1][2] * *z;
	w = a[2][0] * *x + a[2][1] * *y + a[2][2] * *z;
	*x = u;
	*y = v;
	*z = w;
}

void nutequ(double t, double *x, double *y, double *z)
{
	double ls, d, f, n, eps;
	double dpsi, deps, c, s;
	double dx, dy, dz;

	ls = PI2 * frac(0.993133 + 99.997306 * t);	/* mean anomaly of sun */
	d = PI2 * frac(0.827362 + 1236.853087 * t);	/* diff. length moon - sun */
	f = PI2 * frac(0.259089 + 1342.227826 * t);	/* distance between nodes */
	n = PI2 * frac(0.347346 - 5.372447 * t);	/* length of rising node */
	eps = 0.4090928 - 2.2696E-4 * t;	/* Ecliptical inclination  */

	dpsi = (-17.200 * sin(n) - 1.319 * sin(2 * (f - d + n)) - 0.227 * sin(2 * (f + n)) 
		+0.206 * sin(2 * n) + 0.143 * sin(ls)) / ARC;
	deps = (+9.203 * cos(n) + 0.574 * cos(2 * (f - d + n)) + 0.098 * cos(2 * (f + n)) 
		-0.090 * cos(2 * n)) / ARC;
	c = dpsi * cos(eps);
	s = dpsi * sin(eps);

	dx = -(c * *y + s * *z);
	dy = (c * *x - deps * *z);
	dz = (s * *x + deps * *y);

	*x = *x + dx;
	*y = *y + dz;
	*z = *z + dz;
}
