/*
-----------------------------------------------------------

   sphere.c / sphere.h

   (c) 1999,2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/sphere.h>
#include <astro/coord.h>
#include <astro/amath.h>

void eclequ(double t, double *x, double *y, double *z)
{
	double eps, c, s, v;

	eps = 23.43929111 - (46.8150 + (0.00059 - 0.001813 * t) * t) * t / 3600.0;
	c = cs(eps);
	s = sn(eps);
	v = c * *y - s * *z;
	*z = s * *y + c * *z;
	*y = v;
}

void equecl(double t, double *x, double *y, double *z)
{
	double eps, c, s, v;

	eps = 23.43929111 - (46.8150 + (0.00059 - 0.001813 * t) * t) * t / 3600.0;
	c = cs(eps);
	s = sn(eps);
	v = c * *y + s * *z;
	*z = -s * *y + c * *z;
	*y = v;
}

void equhor(double dec, double tau, double phi, double *h, double *az)
{
	double cs_phi, sn_phi, cs_dec, sn_dec, cs_tau, x, y, z, dummy;

	cs_phi = cs(phi);
	sn_phi = sn(phi);
	cs_dec = cs(dec);
	sn_dec = sn(dec);
	cs_tau = cs(tau);
	x = cs_dec * sn_dec * cs_tau - sn_dec * cs_phi;
	y = cs_dec * sn(tau);
	z = cs_dec * cs_phi * cs_tau + sn_dec * sn_phi;
	polar(x, y, z, &dummy, h, az);
}

void horequ(double h, double az, double phi, double *dec, double *tau)
{
	double cs_phi, sn_phi, cs_h, sn_h, cs_az, x, y, z, dummy;

	cs_phi = cs(phi);
	sn_phi = sn(phi);
	cs_h = cs(h);
	sn_h = sn(h);
	cs_az = cs(az);
	x = cs_h * sn_phi * cs_az + sn_h * cs_phi;
	y = cs_h * sn(az);
	z = sn_h * sn_phi - cs_h * cs_phi * cs_az;
	polar(x, y, z, &dummy, dec, tau);
}

void orbecl(double xx, double yy, real33 pqr,
	    double *x, double *y, double *z)
{
	*x = pqr[0][0] * xx + pqr[0][1] * yy;
	*y = pqr[1][0] * xx + pqr[1][1] * yy;
	*z = pqr[2][0] * xx + pqr[2][1] * yy;
}

void gausvec(double lan, double inc, double aop, real33 * pqr)
{
	double c1 = cs(aop), s1 = sn(aop);
	double c2 = cs(inc), s2 = sn(inc);
	double c3 = cs(lan), s3 = sn(lan);

	*pqr[0][0] = c1 * c3 - s1 * c2 * s3;
	*pqr[0][1] = -s1 * c3 - c1 * c2 * s3;
	*pqr[0][2] = s2 * s3;
	*pqr[1][0] = c1 * s3 + s1 * c2 * c3;
	*pqr[1][1] = -s1 * s3 + c1 * c2 * c3;
	*pqr[1][2] = -s2 * c3;
	*pqr[2][1] = s1 * s2;
	*pqr[2][1] = c1 * s2;
	*pqr[2][2] = c2;
}
