/*
-----------------------------------------------------------

   sun.c / sun.h

   (c) 2000 ROALTER - software Int.

-----------------------------------------------------------
*/

#include <astro/sun.h>
#include <astro/amath.h>
#include <math.h>

/* globals	*/
double m2, m3, m4, m5, m6;
double d, a, uu, u, v, dl, dr, db;
double c[9], s[9], c3[9], s3[9];

/* addition theorems:

   cos(a1+a2) = cos(a1)cos(a2) - sin(a1)sin(a2)
   sin(a1+a2) = sin(a1)cos(a2) + cos(a1)sin(a2)	*/ 

void sun_addthe(double c1, double s1, double c2, double s2, double *c, double *s)
{
	*c = c1 * c2 - s1 * s2;
	*s = s1 * c2 + c1 * s2;
}

void sun_term(double t, int i1, int i, int it, double dlc, double dls, double drc, double drs, double dbc, double dbs)
{
	if (it == 0)
		sun_addthe(c3[i1], s3[i1], c[i], s[i], &u, &v);
	else {
		u = u * t;
		v = v * t;
	}
	dl = dl + dlc * u + dls * v;
	dr = dr + drc * u + drs * v;
	db = db + dbc * u + dbs * v;
}

/* distortion elements of venus	*/

void sun_pertven(double t)
{
	int i;

	c[0] = 1.0;
	s[0] = 0.0;
	c[1] = cos(m2);
	s[1] = -sin(m2);
	for (i = 1; i < 6; i++)
		sun_addthe(c[i], s[i], c[1], s[1], &c[i + 1], &s[i + 1]);
	sun_term(t, 2, 0, 0, -0.22, 6892.76, -16707.37, -0.54, 0.00, 0.00);
	sun_term(t, 2, 0, 1, -0.06, -17.35, 42.04, -0.15, 0.00, 0.00);
	sun_term(t, 2, 0, 2, -0.01, -0.05, 0.13, -0.02, 0.00, 0.00);
	sun_term(t, 3, 0, 0, 0.00, 71.98, -139.57, 0.00, 0.00, 0.00);
	sun_term(t, 3, 0, 1, 0.00, -0.39, 0.70, 0.00, 0.00, 0.00);
	sun_term(t, 4, 0, 0, 0.00, 1.04, -1.75, 0.00, 0.00, 0.00);
	sun_term(t, 1, 1, 0, 0.03, -0.07, -0.16, -0.07, 0.02, -0.02);
	sun_term(t, 2, 1, 0, 2.35, -4.26, -4.75, -2.64, 0.00, 0.00);
	sun_term(t, 2, 2, 0, -0.10, 0.06, 0.12, 0.20, 0.02, 0.00);
	sun_term(t, 3, 1, 0, -0.06, -0.03, 0.20, -0.01, 0.01, -0.09);
	sun_term(t, 3, 2, 0, -4.70, 2.90, 8.28, 13.42, 0.01, -0.01);
	sun_term(t, 4, 2, 0, 1.80, -1.74, -1.44, -1.57, 0.04, -0.06);
	sun_term(t, 4, 3, 0, -0.67, 0.03, 0.11, 2.43, 0.01, 0.00);
	sun_term(t, 5, 2, 0, 0.03, -0.03, 0.10, 0.09, 0.01, -0.01);
	sun_term(t, 5, 3, 0, 1.51, -0.40, -0.88, -3.36, 0.18, -0.10);
	sun_term(t, 5, 4, 0, -0.19, -0.09, -0.38, 0.77, 0.00, 0.00);
	sun_term(t, 6, 3, 0, 0.76, -0.68, 0.30, 0.37, 0.01, 0.00);
	sun_term(t, 6, 4, 0, -0.14, -0.04, -0.11, 0.43, -0.03, 0.00);
	sun_term(t, 6, 5, 0, -0.05, -0.07, -0.31, 0.21, 0.00, 0.00);
	sun_term(t, 7, 4, 0, 0.15, -0.04, -0.06, -0.21, 0.01, 0.00);
	sun_term(t, 7, 5, 0, -0.03, -0.03, -0.09, 0.09, -0.01, 0.00);
	sun_term(t, 7, 6, 0, 0.00, -0.04, -0.18, 0.02, 0.00, 0.00);
	sun_term(t, 8, 5, 0, -0.12, -0.03, -0.08, 0.31, -0.02, -0.01);
}

/* distortion elements of mars	*/

void sun_pertmar(double t)
{
	int i;

	c[1] = cos(m4);
	s[1] = -sin(m4);
	for (i = 1; i < 8; i++)
		sun_addthe(c[i], s[i], c[1], s[1], &c[i + 1], &s[i + 1]);
	sun_term(t, 2, 1, 0, -0.22, 0.17, -0.21, -0.27, 0.00, 0.00);
	
sun_term(t, 2, 2, 0, -1.66, 0.62, 0.16, 0.28, 0.00, 0.00);
	sun_term(t, 3, 2, 0, 1.96, 0.57, -1.32, 4.55, 0.00, 0.01);
	sun_term(t, 3, 3, 0, 0.40, 0.15, -0.17, 0.46, 0.00, 0.00);
	sun_term(t, 3, 4, 0, 0.53, 0.26, 0.09, -0.22, 0.00, 0.00);
	sun_term(t, 4, 3, 0, 0.05, 0.12, -0.35, 0.15, 0.00, 0.00);
	sun_term(t, 4, 4, 0, -0.13, -0.48, 1.06, -0.29, 0.01, 0.00);
	sun_term(t, 4, 5, 0, -0.04, -0.20, 0.20, -0.04, 0.00, 0.00);
	sun_term(t, 5, 4, 0, 0.00, -0.03, 0.10, 0.04, 0.00, 0.00);
	sun_term(t, 5, 5, 0, 0.05, -0.07, 0.20, 0.14, 0.00, 0.00);
	sun_term(t, 5, 6, 0, -0.10, 0.11, -0.23, -0.22, 0.00, 0.00);
	sun_term(t, 6, 7, 0, -0.05, 0.00, 0.01, -0.14, 0.00, 0.00);
	sun_term(t, 6, 8, 0, 0.05, 0.01, -0.02, 0.10, 0.00, 0.00);
}

/* distortion elements of jupiter	*/

void sun_pertjup(double t)
{
	int i;

	c[1] = cos(m5);
	s[1] = -sin(m5);
	for (i = 1; i < 4; i++)
		sun_addthe(c[i], s[i], c[1], s[1], &c[i + 1], &s[i + 1]);
	sun_term(t, 0, 1, 0, 0.01, 0.07, 0.18, -0.02, 0.00, -0.02);
	sun_term(t, 1, 1, 0, -0.31, 2.58, 0.52, 0.34, 0.02, 0.00);
	sun_term(t, 2, 1, 0, -7.21, -0.06, 0.13, -16.27, 0.00, -0.02);
	sun_term(t, 2, 2, 0, -0.54, -1.52, 3.09, -1.12, 0.01, -0.17);
	sun_term(t, 2, 3, 0, -0.03, -0.21, 0.38, -0.06, 0.00, -0.02);
	sun_term(t, 3, 1, 0, -0.16, 0.05, -0.18, -0.31, 0.01, 0.00);
	sun_term(t, 3, 2, 0, 0.14, -2.73, 9.23, 0.48, 0.00, 0.00);
	sun_term(t, 3, 3, 0, 0.07, -0.55, 1.83, 0.25, 0.01, 0.00);
	sun_term(t, 3, 4, 0, 0.02, -0.08, 0.25, 0.06, 0.00, 0.00);
	sun_term(t, 4, 2, 0, 0.01, -0.07, 0.16, 0.04, 0.00, 0.00);
	sun_term(t, 4, 3, 0, -0.16, -0.03, 0.08, -0.64, 0.00, 0.00);
	sun_term(t, 4, 4, 0, -0.04, -0.01, 0.03, -0.17, 0.00, 0.00);
}

/* distortion elements of saturn	*/

void sun_pertsat(double t)
{
	c[1] = cos(m6);
	s[1] = -sin(m6);
	
	sun_addthe(c[1], s[1], c[1], s[1], &c[2], &s[2]);
	sun_term(t, 1, 1, 0, 0.00, 0.32, 0.01, 0.00, 0.00, 0.00);
	sun_term(t, 2, 1, 0, -0.08, -0.41, 0.97, -0.18, 0.00, -0.01);
	sun_term(t, 2, 2, 0, 0.04, 0.10, -0.23, 0.10, 0.00, 0.00);
	sun_term(t, 3, 2, 0, 0.04, 0.10, -0.35, 0.13, 0.00, 0.00);
}

/* distortion elements of the moon	*/

void sun_pertmoo()
{
	dl = dl + 6.45 * sin(d) - 0.42 * sin(d - a) + 0.18 * sin(d + a) 
	    +0.17 * sin(d - m3) - 0.06 * sin(d + m3);
	dr = dr + 30.76 * cos(d) - 3.06 * cos(d - a) + 0.85 * cos(d + a) 
	    -0.58 * cos(d + m3) + 0.57 * cos(d - m3);
	db = db + 0.576 * sin(uu);
}

void sun(double t, double *l, double *b, double *r)
{
	int i;

	dl = 0.0;
	dr = 0.0;
	db = 0.0;
	m2 = PI2 * frac(0.1387306 + 162.5485917 * t);
	m3 = PI2 * frac(0.9931266 + 99.9973604 * t);
	m4 = PI2 * frac(0.0543250 + 53.1666028 * t);
	m5 = PI2 * frac(0.0551750 + 8.4293972 * t);
	m6 = PI2 * frac(0.8816500 + 3.3938722 * t);
	d = PI2 * frac(0.8274 + 1236.8531 * t);
	a = PI2 * frac(0.3749 + 1325.5524 * t);
	uu = PI2 * frac(0.2591 + 1342.2278 * t);
	c3[1] = 1.0;
	s3[1] = 0.0;
	c3[2] = cos(m3);
	s3[2] = sin(m3);
	c3[0] = c3[2];
	s3[0] = -s3[0];
	for (i = 3; i < 9; i++)
		sun_addthe(c3[i - 1], s3[i - 1], c3[2], s3[2], &c3[i], &s3[i]);
	sun_pertven;
	sun_pertmar;
	sun_pertjup;
	sun_pertsat;
	sun_pertmoo;
	dl = dl + 6.40 * sin(PI2 * (0.6983 + 0.0561 * t))
	    + 1.87 * sin(PI2 * (0.5764 + 0.4174 * t))
	    + 0.27 * sin(PI2 * (0.4189 + 0.3306 * t))
	    + 0.20 * sin(PI2 * (0.3581 + 2.4814 * t));
	*l = 360.0 * frac(0.7859453 + m3 / PI2
			  + ((6191.2 + 1.1 * t) * t + dl) / 1296.0E3);
	*r = 1.0001398 - 0.0000007 * t + dr * 1E-6;
	*b = db / 3600.0;
}
