#include <math.h>
#include <stdio.h>
#include "amath.h"
#include "atime.h"

#define D1 1236.853086
#define D0 0.827361

void improve(double *t, double *b)
{
	double p2 = 6.283185307;
	double arc = 206264.8062;

	double l, ls, d, f, dlm, dls, dlambda;

	l = p2 * frac(0.374897 + 1325.552410 * *t);	/* mittl. Anomalie des Mondes */
	ls = p2 * frac(0.993133 + 99.997361 * *t);	/* mittl. Anomalie Sonne      */
	d = p2 * (frac(0.5 + D0 + D1 * *t) - 0.5);	/* mit.Diff.Laenge Mond-Sonne */
	f = p2 * frac(0.259086 + 1342.227825 * *t);	/* Knotenabstand              */
	/* periodische Stoerungen der Laengen von Mond und Sonne (in ")        */
	dlm = +22640 * sin(l) - 4586 * sin(l - 2 * d) + 2370 * sin(2 * d) + 769 * sin(2 * l)
	    - 668 * sin(ls) - 412 * sin(2 * f) - 212 * sin(2 * l - 2 * d)
	    - 206 * sin(l + ls - 2 * d) + 192 * sin(l + 2 * d) - 165 * sin(ls - 2 * d)
	    - 125 * sin(d) - 110 * sin(l + ls) + 148 * sin(l - ls) - 55 * sin(2 * f - 2 * d);
	dls = +6893 * sin(ls) + 72 * sin(2 * ls);
	/* Differenz der wahren Laengen von Mond und Sonne (in Umlaeufen)    */
	dlambda = d / p2 + (dlm - dls) / 1296000.0;
	/* Korrektur der Neumondzeit */
	*t = *t - dlambda / D1;
	/* ekliptikale Breite B des Mondes (in Grad) */
	*b = (18520.0 * sin(f + dlm / arc) - 526 * sin(f - 2 * d)) / 3600.0;
}

int main()
{
	int day, month, year, year_calc;
	double hour;
	int lunation_0, lunation_i;
	double t_new_moon, mjd_new_moon, b_moon;

	printf("Year :");
	scanf("%d", &year_calc);
	printf("\n");

	printf("   Date      UT     Width\n");
	printf("              h        o\n");

	lunation_0 = (int) (D1 * (year_calc - 2000) / 100);

	for (lunation_i = lunation_0; lunation_i - lunation_0 < 13; lunation_i++) {
		t_new_moon = (lunation_i - D0) / D1;
		improve(&t_new_moon, &b_moon);
		improve(&t_new_moon, &b_moon);

		mjd_new_moon = 36525.0 * t_new_moon + 51544.5;
		caldat(mjd_new_moon, &day, &month, &year, &hour);
		if (year == year_calc)
			printf("%02d/%02d/%04d%6.1f%9.1f\n", month, day, year, hour, b_moon);
	}
	return (0);
}
