#include <map>
#include <string>
#include <stdio.h>
#include <unistd.h>
#include <fstream>
#include <cfg/cfg.hh>

/*
 * converts str (a list of key=values) to a map[key]=value;
 *
 * white spaces are ignored (unless the whole expression is
 * bounded by `"'  
 */
void strToMap (const string &str, map<string,string> &m)
{
	int i=0;
	string l, r;
	int len = str.length();
	
	while (i < str.length()) {
		l.erase();
		r.erase();
		// leading white spaces
		while (str[i]<33) {
			if (str[i]==10) {
				i++;
				goto ins; 
			}
			if (++i>=len) goto ins;
		};

		// left part (key)
		if (str[i]=='"') {
			if (++i>=len) goto ins;
			while (str[i]!='"') { l+=str[i]; if (++i>=len) goto ins; };
			if (str[i]=='"') i++;
		} else {
			while ((str[i]>32) && str[i]!='=')
				{ l+= str[i]; if (++i>=len) goto ins;};
		}

		//cout << "got " << l << endl;

		// leading white spaces
		while (str[i]<33 || str[i]=='=') { 
			if (str[i]==10) {
				i++;
				goto ins; 
			}
			if (++i>=len) goto ins; 
		};

		// right part (value)
		if (str[i]=='"') {
			if (++i>=len) goto ins;
			while (str[i]!='"') { r+=str[i]; if (++i>=len) goto ins;};
			if (str[i]=='"') i++;
		} else {
			while (str[i]>32)
				{ r+= str[i]; if (++i>=len) goto ins; };
		}

	ins:
		if (l.length()) {
			m[l]=r;
		}
	}
}

/*
 * similar to strToMap, except this time we will have
 * paragraphs translated into maps, with their first
 * line interpreted as the key, and the first empty line
 * as the end, like in:
 *
 * key
 * valuevaluevalue
 * valuevaluevalue
 *
 * key2
 * valuue2value2
 *
 * White spaces will not be ignored this time.
 */
void paraToMap (const string &str, map<string,string> &m)
{
	int pos=0, ppos=0;
	string l, r;
	int nll = strlen ("\n");   // <- it should work on msdos-systems too (?)

	while (pos < str.length()) {
		ppos = pos;
		pos = str.find("\n", pos);		
		string sub = str.substr (ppos, pos-ppos);
		if (sub.length()) {
			if (l.length()) m[l] += sub+"\n";
			else l = sub;
		} else {
			if (l.length()) l.erase();
		}
		pos += nll;
	}
}

Config::Config (const string &file)
	:filename(file)
{
	this->load();
}

Config::Config::Config (istream &in)
	:filename("")
{
	this->load(in);
}

Config::~Config ()
{
}

bool Config::load (const string &file)
{
	if (file.length()) filename = file;
	if (!filename.length()) return false;
	#ifdef DEBUG
	cerr << "Config: loading " << filename << endl;
	#endif
	ifstream in (filename.c_str());
	if (in.fail()) {
		perror (filename.c_str());
		return false;
	} else return this->load (in);
}

bool Config::load (istream &in)
{
	char line[1025];
	string thetext;
	while (!(in.getline(line, 1024)).eof()) {
		thetext += line;
		thetext += "\n";
	}
	map<string,string> p;	
	const string &t=thetext;
	paraToMap (t, p);
	string sec = "lost+found";
	for (map<string,string>::const_iterator i=p.begin();
	     i != p.end(); i++) {
		int s = (*i).first.find_first_of ("[");
		int e = (*i).first.find_last_of ("]");
		if (e<(*i).first.length() && s >= 0 && s < e) {
			sec = (*i).first.substr (s+1, e-s-1);
		}
		strToMap ((*i).second, pgs[sec].pairs);
	}
}

bool Config::dump (const string &file)
{
	if (file.length()) filename=file;
	if (!filename.length()) return false;
	ofstream out (filename.c_str());
	this->dump (out);
}

bool Config::dump (ostream &out)
{
	for (map<string,struct cpara>::const_iterator s = pgs.begin();
	     s != pgs.end(); s++) {
		out << "[" << (*s).first << "]" << endl;
		for (map<string,string>::const_iterator p = (*s).second.pairs.begin();
		     p != (*s).second.pairs.end(); p++) {
			out << "\"" << (*p).first << "\" = \"" << (*p).second << "\"" << endl;
		}
		out << endl;
	}
}

string &Config::val(const string &para, const string &key)
{
	return pgs[para].pairs[key];
}

map<string,string> &Config::sec (const string &p)
{
	return pgs[p].pairs;
}

void Config::clear(void)
{
	pgs.clear();
}
