#ifndef __CRONIC_JOBLIST_HH
#define __CRONIC_JOBLIST_HH

#include <cronic/jobdata.hh>

/*
 * The list of jobs.
 *
 * Since Cronic will be used for both small jobs, which have to be
 * executed fast and with a fine time granulation _and_ for large
 * jobs, we will follow a special strategy:
 *
 * The list will decide whether it is runtime-waste to load the data
 * completely or not. Typically, for big data structures saved on disk,
 * it's a waste of time. So, for those, we will only save the start time,
 * the length and the (joblist) module to execute the job with.
 * That should be enough in order to check if the job is to be executed
 * or not. If the job is to be executed, it will be validated (that is 
 * the job data will be loaded).
 */ 
class JobList {
public:
	virtual void init(const string &dir)=0;
	virtual void init(map<string,string> &cfg)=0;
	virtual void cleanup(void)=0;

	virtual void validate(JobData &data)=0;

	virtual bool empty(void)=0;
	virtual bool earliest(JobData &data)=0;
	virtual void discardEarliest(void)=0;

	virtual bool schedule(const JobData &data)=0;       

	virtual void lock(void)=0;
	virtual void unlock(void)=0;
};

#endif /* __CRONIC_JOBLIST_HH */
