#include <cronic/loop.hh>
#include <unistd.h>
#include <stdexcept>
#include <stdio.h>

Loop::Loop (Time slice, JobList *l, JobShell *s)
	:tslice(1000000),jobs(l),shell(s)
{
	tslice = long(slice*1000000.0);
	jobs = l;
}

Loop::~Loop()
{
}

void Loop::delay(void)
{
	usleep(tslice);
}

bool Loop::itr(void)
{
	jobs->lock();
	JobData d;
	bool ret=false;
	cout.flush();
	printf ("\rutc: %3.1f                     ", UTC());
	fflush (stdout);
	if (jobs->earliest(d) && d.start() <= UTC()) {
		jobs->validate(d);
		// we made a copy of the job data and save it in d.
		// now we must discard the job, because if job-length greater
		// than time-slice, we will have multiple executions of the 
		// same job, which is not what we want.
		jobs->discardEarliest();
		if (!shell) throw runtime_error ("Loop::itr: (execute) no shell");
		else shell->exec (d);
		ret = true;
	}
	jobs->unlock();
	return ret;
}

void Loop::run(void)
{
	while (1) {
		if (!this->itr())
			usleep (tslice);
	}
}

void Loop::execute (const JobData &data)
{
	cerr << "Loop::execute: obsolete" << endl;
}
