#include <link/dlloader.hh>

DLL::DLL(const string &p)
	:path(p),lib(NULL),refcnt(0)
{
	if (p.length()) this->load();
}

DLL::~DLL()
{
	this->forceUnload();
}

bool DLL::load(void)
{
	if (lib && !refcnt) this->unload();
	if (!path.length()) return true;
	lib = dlopen (path.c_str(), RTLD_LAZY);
	return lib != NULL;
}

bool DLL::load (const string &p)
{
	if (lib && !this->unload()) return false;
	path = p;
	return this->load();
}

bool DLL::unload(void)
{
	if (refcnt<1) {
		dlclose (lib);
		lib = NULL;
		refcnt = 0;
		return true;
	} else return false;
}

void DLL::forceUnload (void)
{
	if (lib) dlclose(lib);
	refcnt = 0;
}

void *DLL::create (const string& obj)
{
	if (!lib) return NULL;
	Ptr p = fac[obj];
	if (!p.ptr) {
		string sym("factory_");
		sym += obj;
		p.ptr = (void*(*)())dlsym(lib, sym.c_str());
		if (!p.ptr) return NULL;
		refcnt++;
		fac[obj] = p;
		cerr << "DLL: creating (1)" << endl;
		return (p.ptr)();
	} else {
		refcnt++;
		cerr << "DLL: creating (2)" << endl;
		return (p.ptr)();
	}
}

/*
 * DLLoader
 *
 * Mainly loads a symbol from a given dll. If the dll was already loaded,
 * a refcnt is increased. If not, it will be loaded.
 */

bool DLLoader::get_valid_dll(const string &lib, struct dll_str **d)
{
	struct dll_str &dll = libs[lib];
	if (!dll.dll.loaded()) {
		cerr << "trying to load lib" << endl; cerr.flush();
		if (!autoload || !dll.dll.load(lib)) return false;
		#ifdef DEBUG
		cerr << "DLLoader: loaded " << lib << endl;
		#endif
		dll.refcnt=0;
	}
	if (d) *d = &(dll);
	cerr << "having valid dll" << endl; cerr.flush();
	return true;
}

bool DLLoader::load(const string &lib)
{
	return get_valid_dll(lib, NULL);
}

bool DLLoader::unload(const string &name)
{
	dll_str &dll = libs[name];
	if (!dll.refcnt && dll.dll.loaded()) return libs[name].dll.unload();
	else return false;
}

void DLLoader::forceUnload(const string &name)
{
	dll_str &dll = libs[name];
	if (dll.dll.loaded()) {
		libs[name].dll.forceUnload();
	}
}

void *DLLoader::create(const string &lib, const string &sym)
{
	struct dll_str *d;
	if (!get_valid_dll(lib, &d)) return NULL;
	else {
		cerr << "creating symbol" << endl; cerr.flush();
		void *s = d->dll.create(sym);
		if (s) {
			d->refcnt++;
			syms[s] = lib;
		}
		return s;
	}
}

void DLLoader::ignore (void *sym)
{
	string symname = syms[sym];
	if (!symname.length()) return; // the symbol does not exist (anymore?)
	else {
		cerr << "about to kill" << endl; cerr.flush();
		if (--libs[symname].refcnt <= 0 && autounload) {	       
			this->forceUnload(symname);
			#ifdef DEBUG
			cerr << "DLLoader: unloading " << symname << endl;
			#endif
			cerr << "unloading lib" << endl; cerr.flush();
		}
	}
}
