#ifndef __DLLOADER_HH
#define __DLLOADER_HH

#include <string>
#include <map>
#include <dlfcn.h>

/*
 * This class is designed to work with the cronic library
 * that is, it loads a library and allows the creation of
 * an object. for the object, a factory named factory_objname
 * MUST exist and be declared as extern "C"
 */
class DLL {
	class Ptr {
	public:
		void* (*ptr)(void);
		Ptr(void *(*p)(void)=NULL):ptr(p) {};
		// make sure the default ptr is NULL
	};
	string path;
	void *lib;	
	map<string,Ptr> fac;
	int refcnt;

public:
	DLL (const string &p="");
	~DLL();

	bool load(void); /* load the lib */
	bool load(const string &str);
	bool loaded(void) { return lib != NULL; };
	bool unload(void);
	void forceUnload(void);
	const string &name(void) { return path; };

	void *create(const string& obj);
	void decr(void) { refcnt--; };
};

class DLLoader {
	class dll_str {
	public:
		dll_str():refcnt(0) {};
		~dll_str() { dll.forceUnload(); };
		
		int refcnt;
		DLL dll;
	};
	string basepath;
	map<string,struct dll_str> libs; // library list
	map<void*,string> syms;
	bool autoload, autounload;
	
	bool get_valid_dll(const string &lib, struct dll_str **dll);
	
public:
	//DLLoader(const string &bp="") { basepath = bp; };
	DLLoader ():autoload(true),autounload(true) {};
	~DLLoader() {};

 	bool load(const string &lib);
	bool unload(const string &lib);
	void forceUnload (const string &lib);
 	void *create(const string &lib, const string &sym);
	void ignore (void *sym);
};

#endif /* __DLLOADER_HH */
