#include <sky/core/jobshellcore.hh>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <tcpip/client.hh>
#include <sky/user/user.hh>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

JobShellCore::JobShellCore (const string &i)
{
	this->init(i);
}

JobShellCore::~JobShellCore()
{
	this->cleanup();
}

void JobShellCore::init(const string &i)
{
	map<string,string> cfg;
	cfg["drvaddr"] = "127.0.0.1";
	cfg["camaddr"] = "141.24.147.100";
	cfg["drvport"] = "23231";
	cfg["camport"] = "80";
}

void JobShellCore::init (map<string,string> &cfg)
{
	drvaddr = cfg["drvaddr"];
	camaddr = cfg["camaddr"];
	drvport = atoi(cfg["drvport"].c_str());
	camport = atoi(cfg["camport"].c_str());
}

void JobShellCore::cleanup(void)
{
}

bool JobShellCore::check_perm (const string &user, const string &pass)
{
	//if (!user.length() || !pass.length()) return false;
	return skyCheckUser (user.c_str(), pass.c_str());
}

void JobShellCore::fail (const char *fmt, ...)
{
	fprintf (stderr, "%s\n", fmt);
}

typedef const char * cptr;

inline int ATOI(const string &str, int _d)
{
	return str.length() ? atoi(str.c_str()) : _d;
}

inline double ATOF(const string &str, double _d)
{
	return str.length() ? atof(str.c_str()) : _d;
}

bool JobShellCore::exec (const JobData &data)
{
	cout << "JobShellCore: executing at " << data.start() << endl;

//  	mkdir ((RESULTDIR+data.id()).c_str(), 0775); // here we will save the job results

	long int itop, ileft, iwidth, iheight, ivbin, ihbin, iitime;

	itop    = ATOI(d(data,"itop"), -1);
	ileft   = ATOI(d(data,"ileft"), -1);
	iwidth  = ATOI(d(data,"iwidth"), -1);
	iheight = ATOI(d(data,"iheight"), -1);
	ihbin   = ATOI(d(data,"ihbin"), -1);
	ivbin   = ATOI(d(data,"ivbin"), -1);
	iitime  = ATOI(d(data,"iitime"), -1);

	if (!check_perm(d(data,"uname"), d(data,"upass"))) {
		fail ("insufficient permission for user \"%s\" to execute job",
		      d(data,"uname").c_str());
		return false;
	}

//  	if (!cam_lock(d(data,"uname"))) fail ("cannot lock cam");
	if (!drv_lock(d(data,"uname"))) {
		cam_unlock ();
		fail ("cannot lock drive");
		return false;
	}

	if (!drv_work(d(data,"odesc"), d(data,"oroute"))) {
		cam_unlock();
		drv_unlock();
		fail ("drive failed");
		return false;
	}

//  	if (!cam_work(itop, ileft, iwidth, iheight, ihbin, ivbin, iitime,
//  		      (RESULTDIR+data.id()+"/image.jpg").c_str())) {
//  		cam_unlock();
//  		drv_unlock();
//  		fail ("cam failed");
//  		return false;
//  	}
	       
	return true;
}

/*
 * ATM no handshaking, no checking... just connect and assume everything is ok.
 */
bool JobShellCore::drv_lock (const string &desc)
{
	drvcli.disconnect();
	drvcli.connect(drvaddr, drvport);
	if (drvcli.fd() < 0) {
		perror ("connect");
		return false;
	}
	cerr << "Drive socket fd: " << drvcli.fd() << endl;
	return true;
}

/*
 * send that stuff
 */
bool JobShellCore::drv_work (const string &odesc, const string &oroute)
{
	char eom[]="\n\n";
	char initstr[1024];
	int lnr=0;
	
	for (int i=0; i < oroute.length(); i++)
		if (oroute[i]==10) lnr++; // count the lines

	cerr << "we have " << lnr << " lines." << endl;
	
	//send(drvcli.fd(), or.c_str(), or.length(), 0);
	//cout << "<" << oroute << "> status: " << strerror(errno) << endl;
	return true;
}

/*
 * disconnect from drive
 */
void JobShellCore::drv_unlock (void)
{
	drvcli.disconnect();
}

bool JobShellCore::cam_lock (const string &desc)
{
	camcli.connect(camaddr.c_str(), camport);
	return (camcli.fd() > 0);
}

bool JobShellCore::cam_work (int top, int left, int width, int height,
			     int hbin, int vbin, long int itime, const string &output)
{
	if (camcli.fd()<=0) return false;
	char fmt[]="GET /cgi-bin/camreq?"
		"time=%d&"
		"top=%d&"
		"left=%d&"
		"width=%d&"
		"height=%d&"
		"frame=1&"
		"xbin=%d&"
		"ybin=%d&"
		"reset=true&"
		"force=true HTTP/1.0\n\n";
	char msg[300];
#define MAX_REPL 16384
	char repl[MAX_REPL+1];

	snprintf (msg, 270, fmt, itime, top, left, width, height, hbin, vbin);
	send (camcli.fd(), msg, strlen(msg), 0);
	repl[0]=0;
	int rb=0, total=0;
	do {
		total += rb;
		rb = recv (camcli.fd(), &repl[total], MAX_REPL, 0);
	} while (rb > 0);
	repl[total]=0;
	time_t time;
	
	const char *tstr = strstr (repl, " in about ");
	if (!tstr) {
		cerr << repl << endl;
	} else {
		time = atoi (tstr+10);
		cerr << "time to wait: " << time << endl;
	}

	sleep (time);

	snprintf (msg, 270, "GET /cgi-bin/getjpg.sh?id=%d HTTP/1.0\n\n",
		  rand()%0xffff);
#define MAX_RECV 4096
	char buf[MAX_RECV];

	send (camcli.fd(), buf, strlen(buf), 0);

	FILE *f = fopen (output.c_str(), "w");
	if (!f) return false;

	// receive first part (including mime-header)
	rb = recv (camcli.fd(), buf, MAX_RECV, 0);
	char *tmp = strstr (buf, "\n\n");
	if (!tmp) return false;
	tmp += 2;
	
	fwrite (tmp, rb-(tmp-buf), 1, f);
	
	// receive the rest
	rb = 0;
	do {
		rb = recv (camcli.fd(), buf, MAX_RECV, 0);
		fwrite (buf, rb, 1, f);
	} while (rb <= 0);
	
	fclose (f);
}

void JobShellCore::cam_unlock (void)
{
	camcli.disconnect();
}

void *factory_JobShellCore (void)
{
	return new JobShellCore;
}
