#ifndef __SKYCORE_JOBLISTDRV_HH
#define __SKYCORE_JOBLISTDRV_HH

#include <cronic/joblist.hh>
#include <list>
#include <iostream>

/*
 * axes are specified in x y z coordinates, in the
 * 3d-programming way of coordinates. that is, x and z form a
 * horizontal surface, while y points upwards.
 * there's no special reason why. just because.
 */

/*
 * SkyDrive job list
 */
class JobListDrv: public JobList {
	struct list_info {
		double timebase;
		unsigned long memsize;
	};       

#define MAX_NR 25
	struct job {
		double start;
		double len;
		char time[MAX_NR], x[MAX_NR], y[MAX_NR], z[MAX_NR];
	};

	struct data_ring {
		int size, end, pos;
		bool notempty; /* if "true", the list is definitely _not_ empty
				* if "false", we don't know (depends of end==pos).
				*/ 
		struct job *buf;
	};

	/*
	 * shared file layout:
	 *
	 * |------list-info------|------data-ring-------|---------data(pairs)--------|
	 */
		
	unsigned char *memory;
	struct data_ring *ring;
	struct list_info *info;
	int semkey, semid;
	bool fast, ownshf;
	string shfname;
	int shfd;
	string tmp;
	/*
	 * we'll have a ring buffer.
	 * ringpos < ringend: we have ringend-ringpos elements in there.
	 * ringpos == ringend: we have empty buffer
	 * ringpos > ringend: we have an overrun => we have
	 *                    ringsize-ringpos+ringend elements in there
	 *
	 * ringpos always points to the next element to be read (=> pos+1 is empty, or the end).
	 * ringend always points to the first element available.
	 * ringsize gives the size of the ringbuffer (in elements).
	 */

	string &getShfDir (void) { return tmp; };
	void readList (int fd);

	bool create_shfile (int fd,
			    double tb,
			    /*int left, int rite, int mod, int pairs,*/ int jobs);
	bool read_shfile (int fd);

	void dataCls (JobData &c, const struct job *j);
	void dataStr (struct job *j, const JobData &c);

public:
	JobListDrv();
	JobListDrv(const string &dir);
	~JobListDrv();

	void init(const string &dir);
	void init(map<string,string> &init);
	void cleanup(void);

	void validate(JobData &data);
	bool empty(void);
	bool earliest(JobData &data);
	void discardEarliest(void);
	bool schedule(const JobData &data);
	void lock(void);
	void unlock(void);

	void dataCls (JobData &c, const struct job &j);
	void dataStr (struct job &j, const JobData &c);
};

extern "C" {
	JobListDrv *factory_JobListDrv(void);
};

#endif /* __SKYCORE_JOBLISTDRV_HH*/

